@@ -23,6 +23,11 @@ struct action *action_create(const char *action_name);
 bool action_is_valid(struct action *action);
 
 void action_arg_add_str(struct action *action, const char *key, const char *value);
+void action_arg_add_actionlist(struct action *action, const char *key);
+void action_arg_add_querylist(struct action *action, const char *key);
+
+struct wl_list *action_get_actionlist(struct action *action, const char *key);
+struct wl_list *action_get_querylist(struct action *action, const char *key);
 
 void action_arg_from_xml_node(struct action *action, const char *nodename, const char *content);
 
