@@ -2,13 +2,12 @@
 #ifndef LABWC_SCENE_HELPERS_H
 #define LABWC_SCENE_HELPERS_H
 
+#include <stdbool.h>
+
 struct wlr_scene_node;
-struct wlr_scene_rect;
-struct wlr_scene_tree;
 struct wlr_surface;
+struct wlr_scene_output;
 
-struct wlr_scene_rect *lab_wlr_scene_get_rect(struct wlr_scene_node *node);
-struct wlr_scene_tree *lab_scene_tree_from_node(struct wlr_scene_node *node);
 struct wlr_surface *lab_wlr_surface_from_node(struct wlr_scene_node *node);
 
 /**
@@ -18,4 +17,7 @@ struct wlr_surface *lab_wlr_surface_from_node(struct wlr_scene_node *node);
  */
 struct wlr_scene_node *lab_wlr_scene_get_prev_node(struct wlr_scene_node *node);
 
+/* A variant of wlr_scene_output_commit() that respects wlr_output->pending */
+bool lab_wlr_scene_output_commit(struct wlr_scene_output *scene_output);
+
 #endif /* LABWC_SCENE_HELPERS_H */
