@@ -39,6 +39,7 @@
 #include <wlr/types/wlr_drm_lease_v1.h>
 #include <wlr/types/wlr_virtual_pointer_v1.h>
 #include <wlr/types/wlr_virtual_keyboard_v1.h>
+#include <wlr/types/wlr_tearing_control_v1.h>
 #include <wlr/util/log.h>
 #include "config/keybind.h"
 #include "config/rcxml.h"
@@ -72,7 +73,7 @@ struct input {
 
 /*
  * Virtual keyboards should not belong to seat->keyboard_group. As a result we
- * need to be able to ascertain which wlr_keyboard key/modifer events come from
+ * need to be able to ascertain which wlr_keyboard key/modifier events come from
  * and we achieve that by using `struct keyboard` which inherits `struct input`
  * and adds keybord specific listeners and a wlr_keyboard pointer.
  */
@@ -219,7 +220,8 @@ struct server {
 	struct wl_listener xdg_toplevel_decoration;
 #if HAVE_XWAYLAND
 	struct wlr_xwayland *xwayland;
-	struct wl_listener xwayland_ready;
+	struct wl_listener xwayland_server_ready;
+	struct wl_listener xwayland_xwm_ready;
 	struct wl_listener xwayland_new_surface;
 #endif
 
@@ -291,6 +293,7 @@ struct server {
 
 	struct wl_listener output_layout_change;
 	struct wlr_output_manager_v1 *output_manager;
+	struct wl_listener output_manager_test;
 	struct wl_listener output_manager_apply;
 	/*
 	 * While an output layout change is in process, this counter is
@@ -318,6 +321,9 @@ struct server {
 	struct wlr_pointer_constraints_v1 *constraints;
 	struct wl_listener new_constraint;
 
+	struct wlr_tearing_control_manager_v1 *tearing_control;
+	struct wl_listener tearing_new_object;
+
 	/* Set when in cycle (alt-tab) mode */
 	struct osd_state {
 		struct view *cycle_view;
@@ -456,6 +462,7 @@ void seat_set_pressed(struct seat *seat, struct view *view,
 	struct wlr_scene_node *node, struct wlr_surface *surface,
 	struct wlr_surface *toplevel, uint32_t resize_edges);
 void seat_reset_pressed(struct seat *seat);
+void seat_output_layout_changed(struct seat *seat);
 
 void interactive_begin(struct view *view, enum input_mode mode, uint32_t edges);
 void interactive_finish(struct view *view);
@@ -477,6 +484,8 @@ void handle_output_power_manager_set_mode(struct wl_listener *listener,
 	void *data);
 void output_add_virtual(struct server *server, const char *output_name);
 void output_remove_virtual(struct server *server, const char *output_name);
+void output_enable_adaptive_sync(struct wlr_output *output, bool enabled);
+void new_tearing_hint(struct wl_listener *listener, void *data);
 
 void server_init(struct server *server);
 void server_start(struct server *server);
