@@ -76,17 +76,35 @@ struct theme {
 	int osd_window_switcher_item_padding_y;
 	int osd_window_switcher_item_active_border_width;
 
+	int osd_workspace_switcher_boxes_width;
+	int osd_workspace_switcher_boxes_height;
+
 	/* textures */
 	struct lab_data_buffer *button_close_active_unpressed;
 	struct lab_data_buffer *button_maximize_active_unpressed;
+	struct lab_data_buffer *button_restore_active_unpressed;
 	struct lab_data_buffer *button_iconify_active_unpressed;
 	struct lab_data_buffer *button_menu_active_unpressed;
 
 	struct lab_data_buffer *button_close_inactive_unpressed;
 	struct lab_data_buffer *button_maximize_inactive_unpressed;
+	struct lab_data_buffer *button_restore_inactive_unpressed;
 	struct lab_data_buffer *button_iconify_inactive_unpressed;
 	struct lab_data_buffer *button_menu_inactive_unpressed;
 
+	/* hover variants are optional and may be NULL */
+	struct lab_data_buffer *button_close_active_hover;
+	struct lab_data_buffer *button_maximize_active_hover;
+	struct lab_data_buffer *button_restore_active_hover;
+	struct lab_data_buffer *button_iconify_active_hover;
+	struct lab_data_buffer *button_menu_active_hover;
+
+	struct lab_data_buffer *button_close_inactive_hover;
+	struct lab_data_buffer *button_maximize_inactive_hover;
+	struct lab_data_buffer *button_restore_inactive_hover;
+	struct lab_data_buffer *button_iconify_inactive_hover;
+	struct lab_data_buffer *button_menu_inactive_hover;
+
 	struct lab_data_buffer *corner_top_left_active_normal;
 	struct lab_data_buffer *corner_top_right_active_normal;
 	struct lab_data_buffer *corner_top_left_inactive_normal;
