@@ -8,17 +8,12 @@
  * functions should call these functions.
  */
 
-enum z_direction {
-	LAB_TO_FRONT,
-	LAB_TO_BACK,
-};
-
 struct view;
 
 void view_impl_move_to_front(struct view *view);
 void view_impl_move_to_back(struct view *view);
-void view_impl_move_sub_views(struct view *parent, enum z_direction z_direction);
 void view_impl_map(struct view *view);
+void view_impl_unmap(struct view *view);
 
 /*
  * Updates view geometry at commit based on current position/size,
