@@ -15,11 +15,9 @@
 #include <drm_fourcc.h>
 #include "button/button-xbm.h"
 #include "button/common.h"
-#include "common/dir.h"
 #include "common/grab-file.h"
 #include "common/mem.h"
-#include "config/rcxml.h"
-#include "theme.h"
+#include "common/string-helpers.h"
 #include "buffer.h"
 
 enum token_type {
@@ -165,7 +163,7 @@ out:
 }
 
 static uint32_t
-u32(float *rgba)
+argb32(float *rgba)
 {
 	uint32_t r[4] = { 0 };
 	for (int i = 0; i < 4; i++) {
@@ -267,7 +265,7 @@ button_xbm_from_bitmap(const char *bitmap, struct lab_data_buffer **buffer,
 		wlr_buffer_drop(&(*buffer)->base);
 		*buffer = NULL;
 	}
-	color = u32(rgba);
+	color = argb32(rgba);
 	pixmap = parse_xbm_builtin(bitmap, 6);
 	*buffer = buffer_create_wrap(pixmap.data, pixmap.width, pixmap.height,
 		pixmap.width * 4, /* free_on_destroy */ true);
@@ -282,7 +280,10 @@ button_xbm_load(const char *button_name, struct lab_data_buffer **buffer,
 		wlr_buffer_drop(&(*buffer)->base);
 		*buffer = NULL;
 	}
-	color = u32(rgba);
+	if (string_null_or_empty(button_name)) {
+		return;
+	}
+	color = argb32(rgba);
 
 	/* Read file into memory as it's easier to tokenize that way */
 	char filename[4096] = { 0 };
