@@ -1,8 +1,38 @@
 // SPDX-License-Identifier: GPL-2.0-only
 #include <ctype.h>
+#include <stdbool.h>
 #include "common/buf.h"
 #include "common/mem.h"
 
+static void
+buf_add_one_char(struct buf *s, char ch)
+{
+	if (s->alloc <= s->len + 1) {
+		s->alloc = s->alloc * 3 / 2 + 16;
+		s->buf = xrealloc(s->buf, s->alloc);
+	}
+	s->buf[s->len++] = ch;
+	s->buf[s->len] = '\0';
+}
+
+void
+buf_expand_tilde(struct buf *s)
+{
+	struct buf new;
+	buf_init(&new);
+	for (int i = 0 ; i < s->len ; i++) {
+		if (s->buf[i] == '~') {
+			buf_add(&new, getenv("HOME"));
+		} else {
+			buf_add_one_char(&new, s->buf[i]);
+		}
+	}
+	free(s->buf);
+	s->buf = new.buf;
+	s->len = new.len;
+	s->alloc = new.alloc;
+}
+
 static void
 strip_curly_braces(char *s)
 {
@@ -11,10 +41,16 @@ strip_curly_braces(char *s)
 		return;
 	}
 	len -= 2;
-	memcpy(s, s + 1, len);
+	memmove(s, s + 1, len);
 	s[len] = 0;
 }
 
+static bool
+isvalid(char p)
+{
+	return isalnum(p) || p == '_' || p == '{' || p == '}';
+}
+
 void
 buf_expand_shell_variables(struct buf *s)
 {
@@ -24,12 +60,12 @@ buf_expand_shell_variables(struct buf *s)
 	buf_init(&environment_variable);
 
 	for (int i = 0 ; i < s->len ; i++) {
-		if (s->buf[i] == '$') {
+		if (s->buf[i] == '$' && isvalid(s->buf[i+1])) {
 			/* expand environment variable */
 			environment_variable.len = 0;
 			buf_add(&environment_variable, s->buf + i + 1);
 			char *p = environment_variable.buf;
-			while (isalnum(*p) || *p == '_' || *p == '{' || *p == '}') {
+			while (isvalid(*p)) {
 				++p;
 			}
 			*p = '\0';
@@ -39,22 +75,15 @@ buf_expand_shell_variables(struct buf *s)
 			if (p) {
 				buf_add(&new, p);
 			}
-		} else if (s->buf[i] == '~') {
-			/* expand tilde */
-			buf_add(&new, getenv("HOME"));
 		} else {
-			/* just add one character at a time */
-			if (new.alloc <= new.len + 1) {
-				new.alloc = new.alloc * 3 / 2 + 16;
-				new.buf = xrealloc(new.buf, new.alloc);
-			}
-			new.buf[new.len++] = s->buf[i];
-			new.buf[new.len] = '\0';
+			buf_add_one_char(&new, s->buf[i]);
 		}
 	}
 	free(environment_variable.buf);
 	free(s->buf);
 	s->buf = new.buf;
+	s->len = new.len;
+	s->alloc = new.alloc;
 }
 
 void
