@@ -1,5 +1,4 @@
 // SPDX-License-Identifier: GPL-2.0-only
-#include <assert.h>
 #include <cairo.h>
 #include <drm_fourcc.h>
 #include <pango/pangocairo.h>
@@ -8,6 +7,7 @@
 #include <wlr/util/log.h>
 #include "common/font.h"
 #include "common/graphic-helpers.h"
+#include "common/string-helpers.h"
 #include "labwc.h"
 #include "buffer.h"
 
@@ -85,7 +85,7 @@ font_buffer_create(struct lab_data_buffer **buffer, int max_width,
 		max_width = 2;
 	}
 
-	if (!text || !*text) {
+	if (string_null_or_empty(text)) {
 		return;
 	}
 
