@@ -3,13 +3,15 @@
 #include <strings.h>
 
 #include "common/mem.h"
+#include "common/list.h"
+#include "common/string-helpers.h"
 #include "config/libinput.h"
 #include "config/rcxml.h"
 
 static void
 libinput_category_init(struct libinput_category *l)
 {
-	l->type = DEFAULT_DEVICE;
+	l->type = LAB_LIBINPUT_DEVICE_DEFAULT;
 	l->name = NULL;
 	l->pointer_speed = -2;
 	l->natural_scroll = -1;
@@ -21,21 +23,29 @@ libinput_category_init(struct libinput_category *l)
 	l->accel_profile = -1;
 	l->middle_emu = -1;
 	l->dwt = -1;
+	l->click_method = -1;
+	l->send_events_mode = -1;
 }
 
-enum device_type
+enum lab_libinput_device_type
 get_device_type(const char *s)
 {
-	if (!s) {
-		return DEFAULT_DEVICE;
+	if (string_null_or_empty(s)) {
+		return LAB_LIBINPUT_DEVICE_NONE;
+	}
+	if (!strcasecmp(s, "default")) {
+		return LAB_LIBINPUT_DEVICE_DEFAULT;
 	}
 	if (!strcasecmp(s, "touch")) {
-		return TOUCH_DEVICE;
+		return LAB_LIBINPUT_DEVICE_TOUCH;
+	}
+	if (!strcasecmp(s, "touchpad")) {
+		return LAB_LIBINPUT_DEVICE_TOUCHPAD;
 	}
 	if (!strcasecmp(s, "non-touch")) {
-		return NON_TOUCH_DEVICE;
+		return LAB_LIBINPUT_DEVICE_NON_TOUCH;
 	}
-	return DEFAULT_DEVICE;
+	return LAB_LIBINPUT_DEVICE_NONE;
 }
 
 struct libinput_category *
@@ -43,20 +53,21 @@ libinput_category_create(void)
 {
 	struct libinput_category *l = znew(*l);
 	libinput_category_init(l);
-	wl_list_insert(&rc.libinput_categories, &l->link);
+	wl_list_append(&rc.libinput_categories, &l->link);
 	return l;
 }
 
-/*
- * The default category is the first one with type == DEFAULT_DEVICE and
- * no name. After rcxml_read(), a default category always exists.
- */
+/* After rcxml_read(), a default category always exists. */
 struct libinput_category *
 libinput_category_get_default(void)
 {
 	struct libinput_category *l;
-	wl_list_for_each(l, &rc.libinput_categories, link) {
-		if (l->type == DEFAULT_DEVICE && !l->name) {
+	/*
+	 * Iterate in reverse to get the last one added in case multiple
+	 * 'default' profiles were created.
+	 */
+	wl_list_for_each_reverse(l, &rc.libinput_categories, link) {
+		if (l->type == LAB_LIBINPUT_DEVICE_DEFAULT) {
 			return l;
 		}
 	}
