@@ -1,8 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0-only
 #define _POSIX_C_SOURCE 200809L
 #include <assert.h>
-#include <ctype.h>
-#include <errno.h>
 #include <fcntl.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
@@ -16,6 +14,7 @@
 #include <wlr/util/box.h>
 #include <wlr/util/log.h>
 #include "action.h"
+#include "common/dir.h"
 #include "common/list.h"
 #include "common/macros.h"
 #include "common/mem.h"
@@ -25,6 +24,7 @@
 #include "config/keybind.h"
 #include "config/libinput.h"
 #include "config/mousebind.h"
+#include "config/tablet.h"
 #include "config/rcxml.h"
 #include "labwc.h"
 #include "regions.h"
@@ -36,6 +36,7 @@ static bool in_regions;
 static bool in_usable_area_override;
 static bool in_keybind;
 static bool in_mousebind;
+static bool in_touch;
 static bool in_libinput_category;
 static bool in_window_switcher_field;
 static bool in_window_rules;
@@ -46,6 +47,7 @@ static bool in_action_else_branch;
 static struct usable_area_override *current_usable_area_override;
 static struct keybind *current_keybind;
 static struct mousebind *current_mousebind;
+static struct touch_config_entry *current_touch;
 static struct libinput_category *current_libinput_category;
 static const char *current_mouse_context;
 static struct action *current_keybind_action;
@@ -200,6 +202,8 @@ fill_window_switcher_field(char *nodename, char *content)
 		} else if (!strcmp(content, "app_id")) {
 			wlr_log(WLR_ERROR, "window-switcher field 'app_id' is deprecated");
 			current_field->content = LAB_FIELD_IDENTIFIER;
+		} else if (!strcmp(content, "trimmed_identifier")) {
+			current_field->content = LAB_FIELD_TRIMMED_IDENTIFIER;
 		} else if (!strcmp(content, "title")) {
 			current_field->content = LAB_FIELD_TITLE;
 		} else {
@@ -426,6 +430,22 @@ fill_mousebind(char *nodename, char *content)
 	}
 }
 
+static void
+fill_touch(char *nodename, char *content)
+{
+	if (!strcasecmp(nodename, "touch")) {
+		current_touch = znew(*current_touch);
+		wl_list_append(&rc.touch_configs, &current_touch->link);
+	} else if (!strcasecmp(nodename, "deviceName.touch")) {
+		current_touch->device_name = xstrdup(content);
+	} else if (!strcasecmp(nodename, "mapToOutput.touch")) {
+		current_touch->output_name = xstrdup(content);
+	} else {
+		wlr_log(WLR_ERROR, "Unexpected data in touch parser: %s=\"%s\"",
+			nodename, content);
+	}
+}
+
 static int
 get_accel_profile(const char *s)
 {
@@ -441,10 +461,38 @@ get_accel_profile(const char *s)
 	return -1;
 }
 
+static int
+get_send_events_mode(const char *s)
+{
+	if (!s) {
+		goto err;
+	}
+
+	int ret = parse_bool(s, -1);
+	if (ret >= 0) {
+		return ret
+			? LIBINPUT_CONFIG_SEND_EVENTS_ENABLED
+			: LIBINPUT_CONFIG_SEND_EVENTS_DISABLED;
+	}
+
+	if (!strcasecmp(s, "disabledOnExternalMouse")) {
+		return LIBINPUT_CONFIG_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE;
+	}
+
+err:
+	wlr_log(WLR_INFO, "Not a recognised send events mode");
+	return -1;
+}
+
 static void
 fill_libinput_category(char *nodename, char *content)
 {
-	if (!strcmp(nodename, "category.device.libinput")) {
+	/*
+	 * Create a new profile (libinput-category) on `<libinput><device>`
+	 * so that the 'default' profile can be created without even providing a
+	 * category="" attribute (same as <device category="default">...)
+	 */
+	if (!strcmp(nodename, "device.libinput")) {
 		current_libinput_category = libinput_category_create();
 	}
 
@@ -459,11 +507,19 @@ fill_libinput_category(char *nodename, char *content)
 	string_truncate_at_pattern(nodename, ".device.libinput");
 
 	if (!strcmp(nodename, "category")) {
-		if (!strcmp(content, "touch")
-				|| !strcmp(content, "non-touch")
-				|| !strcmp(content, "default")) {
-			current_libinput_category->type = get_device_type(content);
-		} else {
+		/*
+		 * First we try to get a type based on a number of pre-defined
+		 * terms, for example: 'default', 'touch', 'touchpad' and
+		 * 'non-touch'
+		 */
+		current_libinput_category->type = get_device_type(content);
+
+		/*
+		 * If we couldn't match against any of those terms, we use the
+		 * provided value to define the device name that the settings
+		 * should be applicable to.
+		 */
+		if (current_libinput_category->type == LAB_LIBINPUT_DEVICE_NONE) {
 			current_libinput_category->name = xstrdup(content);
 		}
 	} else if (!strcasecmp(nodename, "naturalScroll")) {
@@ -530,6 +586,22 @@ fill_libinput_category(char *nodename, char *content)
 		current_libinput_category->dwt = ret
 			? LIBINPUT_CONFIG_DWT_ENABLED
 			: LIBINPUT_CONFIG_DWT_DISABLED;
+	} else if (!strcasecmp(nodename, "clickMethod")) {
+		if (!strcasecmp(content, "none")) {
+			current_libinput_category->click_method =
+				LIBINPUT_CONFIG_CLICK_METHOD_NONE;
+		} else if (!strcasecmp(content, "clickfinger")) {
+			current_libinput_category->click_method =
+				LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER;
+		} else if (!strcasecmp(content, "buttonAreas")) {
+			current_libinput_category->click_method =
+				LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS;
+		} else {
+			wlr_log(WLR_ERROR, "invalid clickMethod");
+		}
+	} else if (!strcasecmp(nodename, "sendEventsMode")) {
+		current_libinput_category->send_events_mode =
+			get_send_events_mode(content);
 	}
 }
 
@@ -608,12 +680,29 @@ enum_font_place(const char *place)
 	return FONT_PLACE_UNKNOWN;
 }
 
+static void
+set_adaptive_sync_mode(const char *str, enum adaptive_sync_mode *variable)
+{
+	if (!strcasecmp(str, "fullscreen")) {
+		*variable = LAB_ADAPTIVE_SYNC_FULLSCREEN;
+	} else {
+		int ret = parse_bool(str, -1);
+		if (ret == 1) {
+			*variable = LAB_ADAPTIVE_SYNC_ENABLED;
+		} else {
+			*variable = LAB_ADAPTIVE_SYNC_DISABLED;
+		}
+	}
+}
+
 static void
 entry(xmlNode *node, char *nodename, char *content)
 {
 	/* current <theme><font place=""></font></theme> */
 	static enum font_place font_place = FONT_PLACE_NONE;
 
+	static uint32_t button_map_from;
+
 	if (!nodename) {
 		return;
 	}
@@ -655,8 +744,13 @@ entry(xmlNode *node, char *nodename, char *content)
 			fill_mousebind(nodename, content);
 		}
 	}
+	if (in_touch) {
+		fill_touch(nodename, content);
+		return;
+	}
 	if (in_libinput_category) {
 		fill_libinput_category(nodename, content);
+		return;
 	}
 	if (in_regions) {
 		fill_region(nodename, content);
@@ -682,6 +776,11 @@ entry(xmlNode *node, char *nodename, char *content)
 		return;
 	}
 
+	if (!strcasecmp(nodename, "map.tablet")) {
+		button_map_from = UINT32_MAX;
+		return;
+	}
+
 	/* handle the rest */
 	if (!content) {
 		return;
@@ -702,9 +801,26 @@ entry(xmlNode *node, char *nodename, char *content)
 	} else if (!strcmp(nodename, "gap.core")) {
 		rc.gap = atoi(content);
 	} else if (!strcasecmp(nodename, "adaptiveSync.core")) {
-		set_bool(content, &rc.adaptive_sync);
+		set_adaptive_sync_mode(content, &rc.adaptive_sync);
+	} else if (!strcasecmp(nodename, "allowTearing.core")) {
+		set_bool(content, &rc.allow_tearing);
+		if (rc.allow_tearing) {
+			char *no_atomic_env = getenv("WLR_DRM_NO_ATOMIC");
+			if (!no_atomic_env || strcmp(no_atomic_env, "1") != 0) {
+				rc.allow_tearing = false;
+				wlr_log(WLR_ERROR, "tearing requires WLR_DRM_NO_ATOMIC=1");
+			}
+		}
 	} else if (!strcasecmp(nodename, "reuseOutputMode.core")) {
 		set_bool(content, &rc.reuse_output_mode);
+	} else if (!strcmp(nodename, "policy.placement")) {
+		if (!strcmp(content, "automatic")) {
+			rc.placement_policy = LAB_PLACE_AUTOMATIC;
+		} else if (!strcmp(content, "cursor")) {
+			rc.placement_policy = LAB_PLACE_CURSOR;
+		} else {
+			rc.placement_policy = LAB_PLACE_CENTER;
+		}
 	} else if (!strcmp(nodename, "name.theme")) {
 		rc.theme_name = xstrdup(content);
 	} else if (!strcmp(nodename, "cornerradius.theme")) {
@@ -752,10 +868,24 @@ entry(xmlNode *node, char *nodename, char *content)
 		rc.kb_layout_per_window = !strcasecmp(content, "window");
 	} else if (!strcasecmp(nodename, "screenEdgeStrength.resistance")) {
 		rc.screen_edge_strength = atoi(content);
+	} else if (!strcasecmp(nodename, "windowEdgeStrength.resistance")) {
+		rc.window_edge_strength = atoi(content);
 	} else if (!strcasecmp(nodename, "range.snapping")) {
 		rc.snap_edge_range = atoi(content);
 	} else if (!strcasecmp(nodename, "topMaximize.snapping")) {
 		set_bool(content, &rc.snap_top_maximize);
+	} else if (!strcasecmp(nodename, "notifyClient.snapping")) {
+		if (!strcasecmp(content, "always")) {
+			rc.snap_tiling_events_mode = LAB_TILING_EVENTS_ALWAYS;
+		} else if (!strcasecmp(content, "region")) {
+			rc.snap_tiling_events_mode = LAB_TILING_EVENTS_REGION;
+		} else if (!strcasecmp(content, "edge")) {
+			rc.snap_tiling_events_mode = LAB_TILING_EVENTS_EDGE;
+		} else if (!strcasecmp(content, "never")) {
+			rc.snap_tiling_events_mode = LAB_TILING_EVENTS_NEVER;
+		} else {
+			wlr_log(WLR_ERROR, "ignoring invalid value for notifyClient");
+		}
 
 	/* <windowSwitcher show="" preview="" outlines="" /> */
 	} else if (!strcasecmp(nodename, "show.windowSwitcher")) {
@@ -809,6 +939,29 @@ entry(xmlNode *node, char *nodename, char *content)
 		} else {
 			wlr_log(WLR_ERROR, "Invalid value for <resize popupShow />");
 		}
+	} else if (!strcasecmp(nodename, "mapToOutput.tablet")) {
+		rc.tablet.output_name = xstrdup(content);
+	} else if (!strcasecmp(nodename, "rotate.tablet")) {
+		rc.tablet.rotation = tablet_parse_rotation(atoi(content));
+	} else if (!strcasecmp(nodename, "left.area.tablet")) {
+		rc.tablet.box.x = tablet_get_dbl_if_positive(content, "left");
+	} else if (!strcasecmp(nodename, "top.area.tablet")) {
+		rc.tablet.box.y = tablet_get_dbl_if_positive(content, "top");
+	} else if (!strcasecmp(nodename, "width.area.tablet")) {
+		rc.tablet.box.width = tablet_get_dbl_if_positive(content, "width");
+	} else if (!strcasecmp(nodename, "height.area.tablet")) {
+		rc.tablet.box.height = tablet_get_dbl_if_positive(content, "height");
+	} else if (!strcasecmp(nodename, "button.map.tablet")) {
+		button_map_from = tablet_button_from_str(content);
+	} else if (!strcasecmp(nodename, "to.map.tablet")) {
+		if (button_map_from != UINT32_MAX) {
+			uint32_t button_map_to = mousebind_button_from_str(content, NULL);
+			if (button_map_to != UINT32_MAX) {
+				tablet_button_mapping_add(button_map_from, button_map_to);
+			}
+		} else {
+			wlr_log(WLR_ERROR, "Missing 'button' argument for tablet button mapping");
+		}
 	}
 }
 
@@ -866,6 +1019,12 @@ xml_tree_walk(xmlNode *node)
 			in_mousebind = false;
 			continue;
 		}
+		if (!strcasecmp((char *)n->name, "touch")) {
+			in_touch = true;
+			traverse(n);
+			in_touch = false;
+			continue;
+		}
 		if (!strcasecmp((char *)n->name, "device")) {
 			in_libinput_category = true;
 			traverse(n);
@@ -948,9 +1107,12 @@ rcxml_init(void)
 		wl_list_init(&rc.regions);
 		wl_list_init(&rc.window_switcher.fields);
 		wl_list_init(&rc.window_rules);
+		wl_list_init(&rc.touch_configs);
 	}
 	has_run = true;
 
+	rc.placement_policy = LAB_PLACE_CENTER;
+
 	rc.xdg_shell_server_side_deco = true;
 	rc.ssd_keep_border = true;
 	rc.corner_radius = 8;
@@ -966,14 +1128,22 @@ rcxml_init(void)
 
 	rc.doubleclick_time = 500;
 	rc.scroll_factor = 1.0;
+
+	rc.tablet.output_name = NULL;
+	rc.tablet.rotation = 0;
+	rc.tablet.box = (struct wlr_fbox){0};
+	tablet_load_default_button_mappings();
+
 	rc.repeat_rate = 25;
 	rc.repeat_delay = 600;
 	rc.kb_numlock_enable = true;
 	rc.kb_layout_per_window = false;
 	rc.screen_edge_strength = 20;
+	rc.window_edge_strength = 20;
 
 	rc.snap_edge_range = 1;
 	rc.snap_top_maximize = true;
+	rc.snap_tiling_events_mode = LAB_TILING_EVENTS_ALWAYS;
 
 	rc.window_switcher.show = true;
 	rc.window_switcher.preview = true;
@@ -1077,6 +1247,10 @@ static struct mouse_combos {
 	{ "Frame", "A-Right", "Drag", "Resize", NULL, NULL},
 	{ "Titlebar", "Left", "Press", "Focus", NULL, NULL},
 	{ "Titlebar", "Left", "Press", "Raise", NULL, NULL},
+	{ "Titlebar", "Up", "Scroll", "Unfocus", NULL, NULL},
+	{ "Titlebar", "Up", "Scroll", "Shade", NULL, NULL},
+	{ "Titlebar", "Down", "Scroll", "Unshade", NULL, NULL},
+	{ "Titlebar", "Down", "Scroll", "Focus", NULL, NULL},
 	{ "Title", "Left", "Drag", "Move", NULL, NULL },
 	{ "Title", "Left", "DoubleClick", "ToggleMaximize", NULL, NULL },
 	{ "TitleBar", "Right", "Click", "Focus", NULL, NULL},
@@ -1220,7 +1394,7 @@ static struct {
 	int width;
 } fields[] = {
 	{ LAB_FIELD_TYPE, 25 },
-	{ LAB_FIELD_IDENTIFIER, 25 },
+	{ LAB_FIELD_TRIMMED_IDENTIFIER, 25 },
 	{ LAB_FIELD_TITLE, 50 },
 	{ LAB_FIELD_NONE, 0 },
 };
@@ -1276,6 +1450,8 @@ post_processing(void)
 	if (!libinput_category_get_default()) {
 		/* So we still allow tap to click by default */
 		struct libinput_category *l = libinput_category_create();
+		/* Prevents unused variable warning when compiled without asserts */
+		(void)l;
 		assert(l && libinput_category_get_default() == l);
 	}
 
@@ -1380,69 +1556,68 @@ validate(void)
 	validate_actions();
 }
 
-static void
-rcxml_path(char *buf, size_t len)
-{
-	if (!rc.config_dir) {
-		return;
-	}
-	snprintf(buf, len, "%s/rc.xml", rc.config_dir);
-}
-
-static void
-find_config_file(char *buffer, size_t len, const char *filename)
-{
-	if (filename) {
-		snprintf(buffer, len, "%s", filename);
-		return;
-	}
-	rcxml_path(buffer, len);
-}
-
 void
 rcxml_read(const char *filename)
 {
-	FILE *stream;
-	char *line = NULL;
-	size_t len = 0;
-	struct buf b;
-	static char rcxml[4096] = {0};
-
 	rcxml_init();
 
-	/*
-	 * rcxml_read() can be called multiple times, but we only set rcxml[]
-	 * the first time. The specified 'filename' is only respected the first
-	 * time.
-	 */
-	if (rcxml[0] == '\0') {
-		find_config_file(rcxml, sizeof(rcxml), filename);
-	}
-	if (rcxml[0] == '\0') {
-		wlr_log(WLR_INFO, "cannot find rc.xml config file");
-		goto no_config;
+	struct wl_list paths;
+
+	if (filename) {
+		/* Honour command line argument -c <filename> */
+		wl_list_init(&paths);
+		struct path *path = znew(*path);
+		path->string = xstrdup(filename);
+		wl_list_append(&paths, &path->link);
+	} else {
+		paths_config_create(&paths, "rc.xml");
 	}
 
 	/* Reading file into buffer before parsing - better for unit tests */
-	stream = fopen(rcxml, "r");
-	if (!stream) {
-		wlr_log(WLR_ERROR, "cannot read (%s)", rcxml);
-		goto no_config;
-	}
-	wlr_log(WLR_INFO, "read config file %s", rcxml);
-	buf_init(&b);
-	while (getline(&line, &len, stream) != -1) {
-		char *p = strrchr(line, '\n');
-		if (p) {
-			*p = '\0';
+	struct buf b;
+
+	bool should_merge_config = rc.merge_config;
+	struct wl_list *(*iter)(struct wl_list *list);
+	iter = should_merge_config ? paths_get_prev : paths_get_next;
+
+	/*
+	 * This is the equivalent of a wl_list_for_each() which optionally
+	 * iterates in reverse depending on 'should_merge_config'
+	 *
+	 * If not merging, we iterate forwards and break after the first
+	 * iteration.
+	 *
+	 * If merging, we iterate backwards (least important XDG Base Dir first)
+	 * and keep going.
+	 */
+	for (struct wl_list *elm = iter(&paths); elm != &paths; elm = iter(elm)) {
+		struct path *path = wl_container_of(elm, path, link);
+		FILE *stream = fopen(path->string, "r");
+		if (!stream) {
+			continue;
 		}
-		buf_add(&b, line);
-	}
-	free(line);
-	fclose(stream);
-	rcxml_parse_xml(&b);
-	free(b.buf);
-no_config:
+
+		wlr_log(WLR_INFO, "read config file %s", path->string);
+
+		buf_init(&b);
+		char *line = NULL;
+		size_t len = 0;
+		while (getline(&line, &len, stream) != -1) {
+			char *p = strrchr(line, '\n');
+			if (p) {
+				*p = '\0';
+			}
+			buf_add(&b, line);
+		}
+		zfree(line);
+		fclose(stream);
+		rcxml_parse_xml(&b);
+		zfree(b.buf);
+		if (!should_merge_config) {
+			break;
+		}
+	};
+	paths_destroy(&paths);
 	post_processing();
 	validate();
 }
@@ -1478,6 +1653,16 @@ rcxml_finish(void)
 		zfree(m);
 	}
 
+	struct touch_config_entry *touch_config, *touch_config_tmp;
+	wl_list_for_each_safe(touch_config, touch_config_tmp, &rc.touch_configs, link) {
+		wl_list_remove(&touch_config->link);
+		zfree(touch_config->device_name);
+		zfree(touch_config->output_name);
+		zfree(touch_config);
+	}
+
+	zfree(rc.tablet.output_name);
+
 	struct libinput_category *l, *l_tmp;
 	wl_list_for_each_safe(l, l_tmp, &rc.libinput_categories, link) {
 		wl_list_remove(&l->link);
@@ -1509,6 +1694,7 @@ rcxml_finish(void)
 	current_usable_area_override = NULL;
 	current_keybind = NULL;
 	current_mousebind = NULL;
+	current_touch = NULL;
 	current_libinput_category = NULL;
 	current_mouse_context = NULL;
 	current_keybind_action = NULL;
