@@ -245,12 +245,16 @@ process_cursor_resize(struct server *server, uint32_t time)
 	struct wlr_box new_view_geo = view->current;
 
 	if (server->resize_edges & WLR_EDGE_TOP) {
+		/* Shift y to anchor bottom edge when resizing top */
+		new_view_geo.y = server->grab_box.y + dy;
 		new_view_geo.height = server->grab_box.height - dy;
 	} else if (server->resize_edges & WLR_EDGE_BOTTOM) {
 		new_view_geo.height = server->grab_box.height + dy;
 	}
 
 	if (server->resize_edges & WLR_EDGE_LEFT) {
+		/* Shift x to anchor right edge when resizing left */
+		new_view_geo.x = server->grab_box.x + dx;
 		new_view_geo.width = server->grab_box.width - dx;
 	} else if (server->resize_edges & WLR_EDGE_RIGHT) {
 		new_view_geo.width = server->grab_box.width + dx;
@@ -260,13 +264,13 @@ process_cursor_resize(struct server *server, uint32_t time)
 	view_adjust_size(view, &new_view_geo.width, &new_view_geo.height);
 
 	if (server->resize_edges & WLR_EDGE_TOP) {
-		/* anchor bottom edge */
+		/* After size adjustments, make sure to anchor bottom edge */
 		new_view_geo.y = server->grab_box.y +
 			server->grab_box.height - new_view_geo.height;
 	}
 
 	if (server->resize_edges & WLR_EDGE_LEFT) {
-		/* anchor right edge */
+		/* After size adjustments, make sure to anchor bottom right */
 		new_view_geo.x = server->grab_box.x +
 			server->grab_box.width - new_view_geo.width;
 	}
@@ -471,7 +475,12 @@ cursor_update_common(struct server *server, struct cursor_context *ctx,
 		 */
 		wlr_seat_pointer_notify_clear_focus(wlr_seat);
 		if (!seat->drag.active) {
-			cursor_set(seat, cursor_get_from_ssd(ctx->type));
+			enum lab_cursors cursor = cursor_get_from_ssd(ctx->type);
+			if (ctx->view && ctx->view->shaded && cursor > LAB_CURSOR_GRAB) {
+				/* Prevent resize cursor on borders for shaded SSD */
+				cursor = LAB_CURSOR_DEFAULT;
+			}
+			cursor_set(seat, cursor);
 		}
 	}
 }
@@ -607,6 +616,8 @@ handle_constraint_commit(struct wl_listener *listener, void *data)
 {
 	struct seat *seat = wl_container_of(listener, seat, constraint_commit);
 	struct wlr_pointer_constraint_v1 *constraint = seat->current_constraint;
+	/* Prevents unused variable warning when compiled without asserts */
+	(void)constraint;
 	assert(constraint->surface = data);
 }
 
@@ -924,7 +935,8 @@ handle_press_mousebinding(struct server *server, struct cursor_context *ctx,
 static bool close_menu;
 
 static void
-cursor_button_press(struct seat *seat, struct wlr_pointer_button_event *event)
+cursor_button_press(struct seat *seat, uint32_t button,
+		enum wlr_button_state button_state, uint32_t time_msec)
 {
 	struct server *server = seat->server;
 	struct cursor_context ctx = get_cursor_context(server);
@@ -939,13 +951,11 @@ cursor_button_press(struct seat *seat, struct wlr_pointer_button_event *event)
 	}
 
 	if (server->input_mode == LAB_INPUT_STATE_MENU) {
-		/* We are closing the menu on RELEASE to not leak a stray release */
-		if (ctx.type != LAB_SSD_MENU) {
-			close_menu = true;
-		} else if (menu_call_actions(ctx.node)) {
-			/* Action was successfull, may fail if item just opens a submenu */
-			close_menu = true;
-		}
+		/*
+		 * We close the menu on RELEASE to not leak a stray releases and
+		 * to be consistent with Openbox
+		 */
+		close_menu = true;
 		return;
 	}
 
@@ -989,17 +999,18 @@ cursor_button_press(struct seat *seat, struct wlr_pointer_button_event *event)
 
 	/* Bindings to the Frame context swallow mouse events if activated */
 	bool consumed_by_frame_context =
-		handle_press_mousebinding(server, &ctx, event->button, resize_edges);
+		handle_press_mousebinding(server, &ctx, button, resize_edges);
 
 	if (ctx.surface && !consumed_by_frame_context) {
 		/* Notify client with pointer focus of button press */
-		wlr_seat_pointer_notify_button(seat->seat, event->time_msec,
-			event->button, event->state);
+		wlr_seat_pointer_notify_button(seat->seat, time_msec,
+			button, button_state);
 	}
 }
 
 static void
-cursor_button_release(struct seat *seat, struct wlr_pointer_button_event *event)
+cursor_button_release(struct seat *seat, uint32_t button,
+		enum wlr_button_state button_state, uint32_t time_msec)
 {
 	struct server *server = seat->server;
 	struct cursor_context ctx = get_cursor_context(server);
@@ -1009,9 +1020,13 @@ cursor_button_release(struct seat *seat, struct wlr_pointer_button_event *event)
 
 	if (server->input_mode == LAB_INPUT_STATE_MENU) {
 		if (close_menu) {
-			menu_close_root(server);
-			cursor_update_common(server, &ctx, event->time_msec,
-				/*cursor_has_moved*/ false);
+			if (ctx.type == LAB_SSD_MENU) {
+				menu_call_selected_actions(server);
+			} else {
+				menu_close_root(server);
+				cursor_update_common(server, &ctx, time_msec,
+					/*cursor_has_moved*/ false);
+			}
 			close_menu = false;
 		}
 		return;
@@ -1023,8 +1038,8 @@ cursor_button_release(struct seat *seat, struct wlr_pointer_button_event *event)
 
 		if (pressed_surface) {
 			/* Ensure CSD clients see the release event */
-			wlr_seat_pointer_notify_button(seat->seat, event->time_msec,
-				event->button, event->state);
+			wlr_seat_pointer_notify_button(seat->seat, time_msec,
+				button, button_state);
 		}
 		return;
 	}
@@ -1034,19 +1049,19 @@ cursor_button_release(struct seat *seat, struct wlr_pointer_button_event *event)
 		 * Button released but originally pressed over a different surface.
 		 * Just send the release event to the still focused surface.
 		 */
-		wlr_seat_pointer_notify_button(seat->seat, event->time_msec,
-			event->button, event->state);
+		wlr_seat_pointer_notify_button(seat->seat, time_msec,
+			button, button_state);
 		return;
 	}
 
 	/* Bindings to the Frame context swallow mouse events if activated */
 	bool consumed_by_frame_context =
-		handle_release_mousebinding(server, &ctx, event->button);
+		handle_release_mousebinding(server, &ctx, button);
 
 	if (ctx.surface && !consumed_by_frame_context) {
 		/* Notify client with pointer focus of button release */
-		wlr_seat_pointer_notify_button(seat->seat, event->time_msec,
-			event->button, event->state);
+		wlr_seat_pointer_notify_button(seat->seat, time_msec,
+			button, button_state);
 	}
 }
 
@@ -1063,10 +1078,52 @@ cursor_button(struct wl_listener *listener, void *data)
 
 	switch (event->state) {
 	case WLR_BUTTON_PRESSED:
-		cursor_button_press(seat, event);
+		cursor_button_press(seat, event->button, event->state, event->time_msec);
 		break;
 	case WLR_BUTTON_RELEASED:
-		cursor_button_release(seat, event);
+		cursor_button_release(seat, event->button, event->state, event->time_msec);
+		break;
+	}
+}
+
+void
+cursor_emulate_move_absolute(struct seat *seat, struct wlr_input_device *device,
+		double x, double y, uint32_t time_msec)
+{
+	double lx, ly;
+	wlr_cursor_absolute_to_layout_coords(seat->cursor,
+		device, x, y, &lx, &ly);
+
+	double dx = lx - seat->cursor->x;
+	double dy = ly - seat->cursor->y;
+
+	if (!dx && !dy) {
+		wlr_log(WLR_DEBUG, "dropping useless cursor_emulate: %.10f,%.10f", dx, dy);
+		return;
+	}
+
+	wlr_relative_pointer_manager_v1_send_relative_motion(
+		seat->server->relative_pointer_manager,
+		seat->seat, (uint64_t)time_msec * 1000,
+		dx, dy, dx, dy);
+
+	wlr_cursor_move(seat->cursor, device, dx, dy);
+	process_cursor_motion(seat->server, time_msec);
+	wlr_seat_pointer_notify_frame(seat->seat);
+}
+
+void
+cursor_emulate_button(struct seat *seat, uint32_t button,
+		enum wlr_button_state state, uint32_t time_msec)
+{
+	idle_manager_notify_activity(seat->seat);
+
+	switch (state) {
+	case WLR_BUTTON_PRESSED:
+		cursor_button_press(seat, button, state, time_msec);
+		break;
+	case WLR_BUTTON_RELEASED:
+		cursor_button_release(seat, button, state, time_msec);
 		break;
 	}
 }
