@@ -1,4 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0-only
+#include "edges.h"
 #include "input/keyboard.h"
 #include "labwc.h"
 #include "regions.h"
@@ -55,7 +56,8 @@ interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 		}
 		if (!view_is_floating(view)) {
 			/*
-			 * Un-maximize and restore natural width/height.
+			 * Un-maximize, unshade and restore natural
+			 * width/height.
 			 * Don't reset tiled state yet since we may want
 			 * to keep it (in the snap-to-maximize case).
 			 */
@@ -66,10 +68,14 @@ interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 			geometry.y = max_move_scale(seat->cursor->y,
 				view->current.y, view->current.height,
 				geometry.height);
+
+			view_set_shade(view, false);
+			view_set_untiled(view);
 			view_restore_to(view, geometry);
 		} else {
 			/* Store natural geometry at start of move */
 			view_store_natural_geometry(view);
+			view_invalidate_last_layout_geometry(view);
 		}
 
 		/* Prevent region snapping when just moving via A-Left mousebind */
@@ -79,13 +85,21 @@ interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 		cursor_set(seat, LAB_CURSOR_GRAB);
 		break;
 	case LAB_INPUT_STATE_RESIZE:
-		if (view->fullscreen || view->maximized == VIEW_AXIS_BOTH) {
+		if (view->shaded || view->fullscreen ||
+				view->maximized == VIEW_AXIS_BOTH) {
 			/*
-			 * We don't allow resizing while fullscreen or
-			 * maximized in both directions.
+			 * We don't allow resizing while shaded,
+			 * fullscreen or maximized in both directions.
 			 */
 			return;
 		}
+
+		/*
+		 * Resizing overrides any attempt to restore window
+		 * geometries altered by layout changes.
+		 */
+		view_invalidate_last_layout_geometry(view);
+
 		/*
 		 * If tiled or maximized in only one direction, reset
 		 * tiled/maximized state but keep the same geometry as
@@ -110,6 +124,9 @@ interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 	if (rc.resize_indicator) {
 		resize_indicator_show(view);
 	}
+	if (rc.window_edge_strength) {
+		edges_calculate_visibility(server, view);
+	}
 }
 
 /* Returns true if view was snapped to any edge */
@@ -184,24 +201,17 @@ snap_to_region(struct view *view)
 void
 interactive_finish(struct view *view)
 {
-	if (view->server->grabbed_view == view) {
-		regions_hide_overlay(&view->server->seat);
-		if (view->server->input_mode == LAB_INPUT_STATE_MOVE) {
-			if (!snap_to_region(view)) {
-				if (!snap_to_edge(view)) {
-					/* Reset tiled state if not snapped */
-					view_set_untiled(view);
-				}
-			}
-		}
-		resize_indicator_hide(view);
-
-		view->server->input_mode = LAB_INPUT_STATE_PASSTHROUGH;
-		view->server->grabbed_view = NULL;
-
-		/* Update focus/cursor image */
-		cursor_update_focus(view->server);
+	if (view->server->grabbed_view != view) {
+		return;
 	}
+
+	if (view->server->input_mode == LAB_INPUT_STATE_MOVE) {
+		if (!snap_to_region(view)) {
+			snap_to_edge(view);
+		}
+	}
+
+	interactive_cancel(view);
 }
 
 /*
@@ -212,11 +222,17 @@ interactive_finish(struct view *view)
 void
 interactive_cancel(struct view *view)
 {
-	if (view->server->grabbed_view == view) {
-		resize_indicator_hide(view);
-		view->server->input_mode = LAB_INPUT_STATE_PASSTHROUGH;
-		view->server->grabbed_view = NULL;
-		/* Update focus/cursor image */
-		cursor_update_focus(view->server);
+	if (view->server->grabbed_view != view) {
+		return;
 	}
+
+	regions_hide_overlay(&view->server->seat);
+
+	resize_indicator_hide(view);
+
+	view->server->input_mode = LAB_INPUT_STATE_PASSTHROUGH;
+	view->server->grabbed_view = NULL;
+
+	/* Update focus/cursor image */
+	cursor_update_focus(view->server);
 }
