@@ -1,7 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0-only
 #define _POSIX_C_SOURCE 200809L
 #include <assert.h>
-#include <ctype.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <stdio.h>
@@ -12,6 +11,7 @@
 #include <wlr/util/log.h>
 #include "action.h"
 #include "common/buf.h"
+#include "common/dir.h"
 #include "common/font.h"
 #include "common/list.h"
 #include "common/match.h"
@@ -549,21 +549,27 @@ err:
 static void
 parse_xml(const char *filename, struct server *server)
 {
-	static char buf[4096] = { 0 };
+	struct wl_list paths;
+	paths_config_create(&paths, filename);
 
-	if (!rc.config_dir) {
-		return;
-	}
-	snprintf(buf, sizeof(buf), "%s/%s", rc.config_dir, filename);
+	bool should_merge_config = rc.merge_config;
+	struct wl_list *(*iter)(struct wl_list *list);
+	iter = should_merge_config ? paths_get_prev : paths_get_next;
 
-	FILE *stream = fopen(buf, "r");
-	if (!stream) {
-		wlr_log(WLR_ERROR, "cannot read %s", buf);
-		return;
+	for (struct wl_list *elm = iter(&paths); elm != &paths; elm = iter(elm)) {
+		struct path *path = wl_container_of(elm, path, link);
+		FILE *stream = fopen(path->string, "r");
+		if (!stream) {
+			return;
+		}
+		wlr_log(WLR_INFO, "read menu file %s", path->string);
+		parse(server, stream);
+		fclose(stream);
+		if (!should_merge_config) {
+			break;
+		}
 	}
-	wlr_log(WLR_INFO, "read menu file %s", buf);
-	parse(server, stream);
-	fclose(stream);
+	paths_destroy(&paths);
 }
 
 static int
@@ -719,6 +725,8 @@ init_windowmenu(struct server *server)
 		fill_item("name.action", "ToggleMaximize");
 		current_item = item_create(menu, _("Fullscreen"), false);
 		fill_item("name.action", "ToggleFullscreen");
+		current_item = item_create(menu, _("Roll up/down"), false);
+		fill_item("name.action", "ToggleShade");
 		current_item = item_create(menu, _("Decorations"), false);
 		fill_item("name.action", "ToggleDecorations");
 		current_item = item_create(menu, _("Always on Top"), false);
@@ -932,9 +940,6 @@ menu_execute_item(struct menuitem *item)
 		return false;
 	}
 
-	actions_run(item->parent->triggered_by_view,
-		item->parent->server, &item->actions, 0);
-
 	/*
 	 * We close the menu here to provide a faster feedback to the user.
 	 * We do that without resetting the input state so src/cursor.c
@@ -943,6 +948,17 @@ menu_execute_item(struct menuitem *item)
 	menu_close(item->parent->server->menu_current);
 	item->parent->server->menu_current = NULL;
 
+	struct server *server = item->parent->server;
+	menu_close_root(server);
+	cursor_update_focus(server);
+
+	/*
+	 * We call the actions after menu_close_root() so that virtual keyboard
+	 * input is sent to the focused_surface instead of being absorbed by the
+	 * menu. Consider for example: `wlrctl keyboard type abc`
+	 */
+	actions_run(item->parent->triggered_by_view, server, &item->actions, 0);
+
 	return true;
 }
 
