@@ -11,6 +11,7 @@
 #include <strings.h>
 #include <wlr/backend/drm.h>
 #include <wlr/backend/headless.h>
+#include <wlr/backend/wayland.h>
 #include <wlr/types/wlr_buffer.h>
 #include <wlr/types/wlr_drm_lease_v1.h>
 #include <wlr/types/wlr_output.h>
@@ -28,6 +29,25 @@
 #include "view.h"
 #include "xwayland.h"
 
+static bool
+get_tearing_preference(struct output *output)
+{
+	struct server *server = output->server;
+
+	/* Never allow tearing when disabled */
+	if (!rc.allow_tearing) {
+		return false;
+	}
+
+	/* Tearing is only allowed for the output with the active view */
+	if (!server->active_view || server->active_view->output != output) {
+		return false;
+	}
+
+	/* If the active view requests tearing, or it is toggled on with action, allow it */
+	return server->active_view->tearing_hint;
+}
+
 static void
 output_frame_notify(struct wl_listener *listener, void *data)
 {
@@ -69,7 +89,9 @@ output_frame_notify(struct wl_listener *listener, void *data)
 		return;
 	}
 
-	wlr_scene_output_commit(output->scene_output, NULL);
+	output->wlr_output->pending.tearing_page_flip =
+		get_tearing_preference(output);
+	lab_wlr_scene_output_commit(output->scene_output);
 
 	struct timespec now = { 0 };
 	clock_gettime(CLOCK_MONOTONIC, &now);
@@ -86,6 +108,7 @@ output_destroy_notify(struct wl_listener *listener, void *data)
 	wl_list_remove(&output->frame.link);
 	wl_list_remove(&output->destroy.link);
 	wl_list_remove(&output->request_state.link);
+	seat_output_layout_changed(&output->server->seat);
 
 	for (size_t i = 0; i < ARRAY_SIZE(output->layer_tree); i++) {
 		wlr_scene_node_destroy(&output->layer_tree[i]->node);
@@ -206,7 +229,7 @@ new_output_notify(struct wl_listener *listener, void *data)
 
 	/*
 	 * Configures the output created by the backend to use our allocator
-	 * and our renderer. Must be done once, before commiting the output
+	 * and our renderer. Must be done once, before committing the output
 	 */
 	if (!wlr_output_init_render(wlr_output, server->allocator,
 			server->renderer)) {
@@ -250,15 +273,8 @@ new_output_notify(struct wl_listener *listener, void *data)
 		}
 	}
 
-	if (rc.adaptive_sync) {
-		wlr_output_enable_adaptive_sync(wlr_output, true);
-		if (!wlr_output_test(wlr_output)) {
-			wlr_output_enable_adaptive_sync(wlr_output, false);
-			wlr_log(WLR_DEBUG,
-				"failed to enable adaptive sync for output %s", wlr_output->name);
-		} else {
-			wlr_log(WLR_INFO, "adaptive sync enabled for output %s", wlr_output->name);
-		}
+	if (rc.adaptive_sync == LAB_ADAPTIVE_SYNC_ENABLED) {
+		output_enable_adaptive_sync(wlr_output, true);
 	}
 
 	wlr_output_commit(wlr_output);
@@ -337,6 +353,7 @@ new_output_notify(struct wl_listener *listener, void *data)
 
 	server->pending_output_layout_change--;
 	do_output_layout_change(server);
+	seat_output_layout_changed(&output->server->seat);
 }
 
 void
@@ -388,10 +405,11 @@ output_update_for_layout_change(struct server *server)
 	cursor_update_image(&server->seat);
 }
 
-static void
+static bool
 output_config_apply(struct server *server,
 		struct wlr_output_configuration_v1 *config)
 {
+	bool success = true;
 	server->pending_output_layout_change++;
 
 	struct wlr_output_configuration_head_v1 *head;
@@ -416,11 +434,18 @@ output_config_apply(struct server *server,
 			}
 			wlr_output_set_scale(o, head->state.scale);
 			wlr_output_set_transform(o, head->state.transform);
-			wlr_output_enable_adaptive_sync(o, head->state.adaptive_sync_enabled);
+			output_enable_adaptive_sync(o, head->state.adaptive_sync_enabled);
 		}
 		if (!wlr_output_commit(o)) {
-			wlr_log(WLR_ERROR, "Output config commit failed");
-			continue;
+			/*
+			 * FIXME: This is only part of the story, we should revert
+			 *        all previously commited outputs as well here.
+			 *
+			 *        See https://github.com/labwc/labwc/pull/1528
+			 */
+			wlr_log(WLR_INFO, "Output config commit failed: %s", o->name);
+			success = false;
+			break;
 		}
 
 		/* Only do Layout specific actions if the commit went trough */
@@ -460,13 +485,94 @@ output_config_apply(struct server *server,
 
 	server->pending_output_layout_change--;
 	do_output_layout_change(server);
+	return success;
 }
 
 static bool
 verify_output_config_v1(const struct wlr_output_configuration_v1 *config)
 {
-	/* TODO implement */
+	const char *err_msg = NULL;
+	struct wlr_output_configuration_head_v1 *head;
+	wl_list_for_each(head, &config->heads, link) {
+		if (!head->state.enabled) {
+			continue;
+		}
+
+		/* Handle custom modes */
+		if (!head->state.mode) {
+			int32_t refresh = head->state.custom_mode.refresh;
+
+			if (wlr_output_is_drm(head->state.output) && refresh == 0) {
+				/*
+				 * wlroots has a bug which causes a divide by zero
+				 * when setting the refresh rate to 0 on a DRM output.
+				 * It is already fixed but not part of an official 0.17.x
+				 * release. Even it would be we still need to carry the
+				 * fix here to prevent older 0.17.x releases from crashing.
+				 *
+				 * https://gitlab.freedesktop.org/wlroots/wlroots/-/issues/3791
+				 */
+				err_msg = "DRM backend does not support a refresh rate of 0";
+				goto custom_mode_failed;
+			}
+
+			if (wlr_output_is_wl(head->state.output) && refresh != 0) {
+				/* Wayland backend does not support refresh rates */
+				err_msg = "Wayland backend refresh rates unsupported";
+				goto custom_mode_failed;
+			}
+		}
+
+		if (wlr_output_is_wl(head->state.output)
+				&& !head->state.adaptive_sync_enabled) {
+			err_msg = "Wayland backend requires adaptive sync";
+			goto custom_mode_failed;
+		}
+
+		/*
+		 * Ensure the new output state can be applied on
+		 * its own and inform the client when it can not.
+		 *
+		 * Applying the changes may still fail later when
+		 * getting mixed with wlr_output->pending which
+		 * may contain further unrelated changes.
+		 */
+		struct wlr_output_state output_state;
+		wlr_output_state_init(&output_state);
+		wlr_output_head_v1_state_apply(&head->state, &output_state);
+
+		if (!wlr_output_test_state(head->state.output, &output_state)) {
+			wlr_output_state_finish(&output_state);
+			return false;
+		}
+		wlr_output_state_finish(&output_state);
+	}
+
 	return true;
+
+custom_mode_failed:
+	assert(err_msg);
+	wlr_log(WLR_INFO, "%s (%s: %dx%d@%d)",
+		err_msg,
+		head->state.output->name,
+		head->state.custom_mode.width,
+		head->state.custom_mode.height,
+		head->state.custom_mode.refresh);
+	return false;
+}
+
+static void
+handle_output_manager_test(struct wl_listener *listener, void *data)
+{
+	struct server *server = wl_container_of(listener, server, output_manager_test);
+	struct wlr_output_configuration_v1 *config = data;
+
+	if (verify_output_config_v1(config)) {
+		wlr_output_configuration_v1_send_succeeded(config);
+	} else {
+		wlr_output_configuration_v1_send_failed(config);
+	}
+	wlr_output_configuration_v1_destroy(config);
 }
 
 static void
@@ -478,8 +584,7 @@ handle_output_manager_apply(struct wl_listener *listener, void *data)
 
 	bool config_is_good = verify_output_config_v1(config);
 
-	if (config_is_good) {
-		output_config_apply(server, config);
+	if (config_is_good && output_config_apply(server, config)) {
 		wlr_output_configuration_v1_send_succeeded(config);
 	} else {
 		wlr_output_configuration_v1_send_failed(config);
@@ -497,7 +602,7 @@ handle_output_manager_apply(struct wl_listener *listener, void *data)
 }
 
 /*
- * Take the way outputs are currently configured/layed out and turn that into
+ * Take the way outputs are currently configured/laid out and turn that into
  * a struct that we send to clients via the wlr_output_configuration v1
  * interface
  */
@@ -587,6 +692,10 @@ output_manager_init(struct server *server)
 	wl_signal_add(&server->output_manager->events.apply,
 		&server->output_manager_apply);
 
+	server->output_manager_test.notify = handle_output_manager_test;
+	wl_signal_add(&server->output_manager->events.test,
+		&server->output_manager_test);
+
 	server->gamma_control_set_gamma.notify = handle_gamma_control_set_gamma;
 	wl_signal_add(&server->gamma_control_manager_v1->events.set_gamma,
 		&server->gamma_control_set_gamma);
@@ -811,3 +920,17 @@ output_remove_virtual(struct server *server, const char *output_name)
 		}
 	}
 }
+
+void
+output_enable_adaptive_sync(struct wlr_output *output, bool enabled)
+{
+	wlr_output_enable_adaptive_sync(output, enabled);
+	if (!wlr_output_test(output)) {
+		wlr_output_enable_adaptive_sync(output, false);
+		wlr_log(WLR_DEBUG,
+				"failed to enable adaptive sync for output %s", output->name);
+	} else {
+		wlr_log(WLR_INFO, "adaptive sync %sabled for output %s",
+			enabled ? "en" : "dis", output->name);
+	}
+}
