@@ -9,6 +9,8 @@
 #include <wlr/types/wlr_touch.h>
 #include <wlr/util/log.h>
 #include "common/mem.h"
+#include "input/tablet.h"
+#include "input/tablet_pad.h"
 #include "input/input.h"
 #include "input/keyboard.h"
 #include "input/key-state.h"
@@ -32,22 +34,81 @@ input_device_destroy(struct wl_listener *listener, void *data)
 	free(input);
 }
 
-static bool
-is_touch_device(struct wlr_input_device *wlr_input_device)
+static enum lab_libinput_device_type
+device_type_from_wlr_device(struct wlr_input_device *wlr_input_device)
 {
 	switch (wlr_input_device->type) {
 	case WLR_INPUT_DEVICE_TOUCH:
 	case WLR_INPUT_DEVICE_TABLET_TOOL:
-		return true;
+		return LAB_LIBINPUT_DEVICE_TOUCH;
 	default:
 		break;
 	}
-	return false;
+
+	if (wlr_input_device->type == WLR_INPUT_DEVICE_POINTER &&
+			wlr_input_device_is_libinput(wlr_input_device)) {
+		struct libinput_device *libinput_device =
+			wlr_libinput_get_device_handle(wlr_input_device);
+
+		if (libinput_device_config_tap_get_finger_count(libinput_device) > 0) {
+			return LAB_LIBINPUT_DEVICE_TOUCHPAD;
+		}
+	}
+
+	return LAB_LIBINPUT_DEVICE_NON_TOUCH;
+}
+
+/*
+ * Get applicable profile (category) by matching first by name and secondly be
+ * type (e.g. 'touch' and 'non-touch'). If not suitable match is found based on
+ * those two criteria we fallback on 'default'.
+ */
+static struct libinput_category *
+get_category(struct wlr_input_device *device)
+{
+	/* By name */
+	struct libinput_category *category;
+	wl_list_for_each_reverse(category, &rc.libinput_categories, link) {
+		if (category->name) {
+			if (!strcasecmp(device->name, category->name)) {
+				return category;
+			}
+		}
+	}
+
+	/* By type */
+	enum lab_libinput_device_type type = device_type_from_wlr_device(device);
+	wl_list_for_each_reverse(category, &rc.libinput_categories, link) {
+		if (category->type == type) {
+			return category;
+		}
+	}
+
+	/* Use default profile as a fallback */
+	return libinput_category_get_default();
 }
 
 static void
 configure_libinput(struct wlr_input_device *wlr_input_device)
 {
+	/*
+	 * TODO: We do not check any return values for the various
+	 *       libinput_device_config_*_set_*() calls. It would
+	 *       be nice if we could inform the users via log file
+	 *       that some libinput setting could not be applied.
+	 *
+	 * TODO: We are currently using int32_t with -1 as default
+	 *       to desribe the not-configured state. This is not
+	 *       really optimal as we can't properly deal with
+	 *       enum values that are 0. After some discussion via
+	 *       IRC the best way forward seem to be to use a
+	 *       uint32_t instead and UINT32_MAX as indicator for
+	 *       a not-configured state. This allows to properly
+	 *       test the enum being a member of a bitset via
+	 *       mask & value == value. All libinput enums are
+	 *       way below UINT32_MAX.
+	 */
+
 	if (!wlr_input_device) {
 		wlr_log(WLR_ERROR, "no wlr_input_device");
 		return;
@@ -63,25 +124,7 @@ configure_libinput(struct wlr_input_device *wlr_input_device)
 		return;
 	}
 
-	enum device_type current_type;
-	current_type = is_touch_device(wlr_input_device)
-			? TOUCH_DEVICE : NON_TOUCH_DEVICE;
-
-	struct libinput_category *device_category, *dc = NULL;
-	wl_list_for_each(device_category, &rc.libinput_categories, link) {
-		if (device_category->name) {
-			if (!strcasecmp(wlr_input_device->name,
-					device_category->name)) {
-				dc = device_category;
-				break;
-			}
-		} else if (device_category->type == current_type) {
-			dc = device_category;
-		} else if (device_category->type == DEFAULT_DEVICE && !dc) {
-			/* Match default category as last-resort */
-			dc = device_category;
-		}
-	}
+	struct libinput_category *dc = get_category(wlr_input_device);
 
 	/*
 	 * The above logic should have always matched SOME category
@@ -164,6 +207,36 @@ configure_libinput(struct wlr_input_device *wlr_input_device)
 		wlr_log(WLR_INFO, "dwt configured");
 		libinput_device_config_dwt_set_enabled(libinput_dev, dc->dwt);
 	}
+
+	if ((dc->click_method != LIBINPUT_CONFIG_CLICK_METHOD_NONE
+			&& (libinput_device_config_click_get_methods(libinput_dev)
+				& dc->click_method) == 0)
+			|| dc->click_method < 0) {
+		wlr_log(WLR_INFO, "click method not configured");
+	} else {
+		wlr_log(WLR_INFO, "click method configured");
+
+		/*
+		 * Note, the documentation claims that:
+		 * > [...] The device may require changing to a neutral state
+		 * > first before activating the new method.
+		 *
+		 * However, just setting the method seems to work without
+		 * issues.
+		 */
+
+		libinput_device_config_click_set_method(libinput_dev, dc->click_method);
+	}
+
+	if ((dc->send_events_mode != LIBINPUT_CONFIG_SEND_EVENTS_ENABLED
+			&& (libinput_device_config_send_events_get_modes(libinput_dev)
+				& dc->send_events_mode) == 0)
+			|| dc->send_events_mode < 0) {
+		wlr_log(WLR_INFO, "send events mode not configured");
+	} else {
+		wlr_log(WLR_INFO, "send events mode configured");
+		libinput_device_config_send_events_set_mode(libinput_dev, dc->send_events_mode);
+	}
 }
 
 static struct wlr_output *
@@ -179,6 +252,25 @@ output_by_name(struct server *server, const char *name)
 	return NULL;
 }
 
+static void
+map_input_to_output(struct seat *seat, struct wlr_input_device *dev, char *output_name)
+{
+	struct wlr_output *output = NULL;
+	if (output_name) {
+		output = output_by_name(seat->server, output_name);
+	}
+	wlr_cursor_map_input_to_output(seat->cursor, dev, output);
+	wlr_cursor_map_input_to_region(seat->cursor, dev, NULL);
+}
+
+static void
+map_pointer_to_output(struct seat *seat, struct wlr_input_device *dev)
+{
+	struct wlr_pointer *pointer = wlr_pointer_from_input_device(dev);
+	wlr_log(WLR_INFO, "map pointer to output %s\n", pointer->output_name);
+	map_input_to_output(seat, dev, pointer->output_name);
+}
+
 static struct input *
 new_pointer(struct seat *seat, struct wlr_input_device *dev)
 {
@@ -189,14 +281,7 @@ new_pointer(struct seat *seat, struct wlr_input_device *dev)
 
 	/* In support of running with WLR_WL_OUTPUTS set to >=2 */
 	if (dev->type == WLR_INPUT_DEVICE_POINTER) {
-		struct wlr_output *output = NULL;
-		struct wlr_pointer *pointer = wlr_pointer_from_input_device(dev);
-		wlr_log(WLR_INFO, "map pointer to output %s\n", pointer->output_name);
-		if (pointer->output_name) {
-			output = output_by_name(seat->server, pointer->output_name);
-		}
-		wlr_cursor_map_input_to_output(seat->cursor, dev, output);
-		wlr_cursor_map_input_to_region(seat->cursor, dev, NULL);
+		map_pointer_to_output(seat, dev);
 	}
 	return input;
 }
@@ -232,6 +317,23 @@ new_keyboard(struct seat *seat, struct wlr_input_device *device, bool virtual)
 	return (struct input *)keyboard;
 }
 
+static void
+map_touch_to_output(struct seat *seat, struct wlr_input_device *dev)
+{
+	struct wlr_touch *touch = wlr_touch_from_input_device(dev);
+
+	char *touch_config_output_name = NULL;
+	struct touch_config_entry *config_entry =
+		touch_find_config_for_device(touch->base.name);
+	if (config_entry) {
+		touch_config_output_name = config_entry->output_name;
+	}
+
+	char *output_name = touch->output_name ? touch->output_name : touch_config_output_name;
+	wlr_log(WLR_INFO, "map touch to output %s\n", output_name ? output_name : "unknown");
+	map_input_to_output(seat, dev, output_name);
+}
+
 static struct input *
 new_touch(struct seat *seat, struct wlr_input_device *dev)
 {
@@ -239,18 +341,32 @@ new_touch(struct seat *seat, struct wlr_input_device *dev)
 	input->wlr_input_device = dev;
 	configure_libinput(dev);
 	wlr_cursor_attach_input_device(seat->cursor, dev);
-
 	/* In support of running with WLR_WL_OUTPUTS set to >=2 */
-	if (dev->type == WLR_INPUT_DEVICE_TOUCH) {
-		struct wlr_output *output = NULL;
-		struct wlr_touch *touch = wlr_touch_from_input_device(dev);
-		wlr_log(WLR_INFO, "map touch to output %s\n", touch->output_name);
-		if (touch->output_name) {
-			output = output_by_name(seat->server, touch->output_name);
-		}
-		wlr_cursor_map_input_to_output(seat->cursor, dev, output);
-		wlr_cursor_map_input_to_region(seat->cursor, dev, NULL);
-	}
+	map_touch_to_output(seat, dev);
+
+	return input;
+}
+
+static struct input *
+new_tablet(struct seat *seat, struct wlr_input_device *dev)
+{
+	struct input *input = znew(*input);
+	input->wlr_input_device = dev;
+	tablet_init(seat, dev);
+	wlr_cursor_attach_input_device(seat->cursor, dev);
+	wlr_log(WLR_INFO, "map tablet to output %s\n", rc.tablet.output_name);
+	map_input_to_output(seat, dev, rc.tablet.output_name);
+
+	return input;
+}
+
+static struct input *
+new_tablet_pad(struct seat *seat, struct wlr_input_device *dev)
+{
+	struct input *input = znew(*input);
+	input->wlr_input_device = dev;
+	tablet_pad_init(seat, dev);
+
 	return input;
 }
 
@@ -306,6 +422,12 @@ new_input_notify(struct wl_listener *listener, void *data)
 	case WLR_INPUT_DEVICE_TOUCH:
 		input = new_touch(seat, device);
 		break;
+	case WLR_INPUT_DEVICE_TABLET_TOOL:
+		input = new_tablet(seat, device);
+		break;
+	case WLR_INPUT_DEVICE_TABLET_PAD:
+		input = new_tablet_pad(seat, device);
+		break;
 	default:
 		wlr_log(WLR_INFO, "unsupported input device");
 		return;
@@ -440,13 +562,16 @@ seat_finish(struct server *server)
 }
 
 static void
-configure_keyboard(struct wlr_input_device *device)
+configure_keyboard(struct seat *seat, struct input *input)
 {
+	struct wlr_input_device *device = input->wlr_input_device;
 	assert(device->type == WLR_INPUT_DEVICE_KEYBOARD);
+	struct keyboard *keyboard = (struct keyboard *)input;
 	struct wlr_keyboard *kb = wlr_keyboard_from_input_device(device);
-	wlr_keyboard_set_repeat_info(kb, rc.repeat_rate, rc.repeat_delay);
+	keyboard_configure(seat, kb, keyboard->is_virtual);
 }
 
+/* This is called on SIGHUP (generally in response to labwc --reconfigure */
 void
 seat_reconfigure(struct server *server)
 {
@@ -455,10 +580,18 @@ seat_reconfigure(struct server *server)
 	wl_list_for_each(input, &seat->inputs, link) {
 		switch (input->wlr_input_device->type) {
 		case WLR_INPUT_DEVICE_KEYBOARD:
-			configure_keyboard(input->wlr_input_device);
+			configure_keyboard(seat, input);
 			break;
 		case WLR_INPUT_DEVICE_POINTER:
 			configure_libinput(input->wlr_input_device);
+			map_pointer_to_output(seat, input->wlr_input_device);
+			break;
+		case WLR_INPUT_DEVICE_TOUCH:
+			configure_libinput(input->wlr_input_device);
+			map_touch_to_output(seat, input->wlr_input_device);
+			break;
+		case WLR_INPUT_DEVICE_TABLET_TOOL:
+			map_input_to_output(seat, input->wlr_input_device, rc.tablet.output_name);
 			break;
 		default:
 			break;
@@ -543,11 +676,15 @@ seat_set_focus_layer(struct seat *seat, struct wlr_layer_surface_v1 *layer)
 static void
 pressed_surface_destroy(struct wl_listener *listener, void *data)
 {
-	struct wlr_surface *surface = data;
 	struct seat *seat = wl_container_of(listener, seat,
 		pressed_surface_destroy);
 
-	assert(surface == seat->pressed.surface);
+	/*
+	 * Using data directly prevents 'unused variable'
+	 * warning when compiling without asserts
+	 */
+	assert(data == seat->pressed.surface);
+
 	seat_reset_pressed(seat);
 }
 
@@ -585,3 +722,24 @@ seat_reset_pressed(struct seat *seat)
 	seat->pressed.toplevel = NULL;
 	seat->pressed.resize_edges = 0;
 }
+
+void
+seat_output_layout_changed(struct seat *seat)
+{
+	struct input *input = NULL;
+	wl_list_for_each(input, &seat->inputs, link) {
+		switch (input->wlr_input_device->type) {
+		case WLR_INPUT_DEVICE_POINTER:
+			map_pointer_to_output(seat, input->wlr_input_device);
+			break;
+		case WLR_INPUT_DEVICE_TOUCH:
+			map_touch_to_output(seat, input->wlr_input_device);
+			break;
+		case WLR_INPUT_DEVICE_TABLET_TOOL:
+			map_input_to_output(seat, input->wlr_input_device, rc.tablet.output_name);
+			break;
+		default:
+			break;
+		}
+	}
+}
