@@ -42,10 +42,10 @@ refocus_output(struct session_lock_output *output)
 
 	struct session_lock_output *iter;
 	wl_list_for_each(iter, &output->lock->session_lock_outputs, link) {
-		if (iter == output || !iter->surface) {
+		if (iter == output || !iter->surface || !iter->surface->surface) {
 			continue;
 		}
-		if (iter->surface->mapped) {
+		if (iter->surface->surface->mapped) {
 			focus_surface(output->lock, iter->surface->surface);
 			return;
 		}
@@ -110,7 +110,7 @@ found_lock_output:
 	wl_signal_add(&lock_surface->events.destroy, &lock_output->surface_destroy);
 
 	lock_output->surface_map.notify = handle_surface_map;
-	wl_signal_add(&lock_surface->events.map, &lock_output->surface_map);
+	wl_signal_add(&lock_surface->surface->events.map, &lock_output->surface_map);
 
 	lock_output_reconfigure(lock_output);
 }
@@ -143,7 +143,7 @@ handle_commit(struct wl_listener *listener, void *data)
 	struct session_lock_output *output = wl_container_of(listener, output, commit);
 	uint32_t require_reconfigure = WLR_OUTPUT_STATE_MODE
 		| WLR_OUTPUT_STATE_SCALE | WLR_OUTPUT_STATE_TRANSFORM;
-	if (event->committed & require_reconfigure) {
+	if (event->state->committed & require_reconfigure) {
 		lock_output_reconfigure(output);
 	}
 }
