@@ -20,7 +20,7 @@ ssd_border_create(struct ssd *ssd)
 	struct view *view = ssd->view;
 	struct theme *theme = view->server->theme;
 	int width = view->current.width;
-	int height = view->current.height;
+	int height = view_effective_height(view, /* use_pending */ false);
 	int full_width = width + 2 * theme->border_width;
 
 	float *color;
@@ -83,7 +83,7 @@ ssd_border_update(struct ssd *ssd)
 	struct theme *theme = view->server->theme;
 
 	int width = view->current.width;
-	int height = view->current.height;
+	int height = view_effective_height(view, /* use_pending */ false);
 	int full_width = width + 2 * theme->border_width;
 
 	struct ssd_part *part;
