@@ -9,10 +9,8 @@
 #include "config.h"
 #include <assert.h>
 #include <cairo.h>
-#include <ctype.h>
 #include <drm_fourcc.h>
 #include <glib.h>
-#include <math.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,6 +23,7 @@
 #include "common/font.h"
 #include "common/graphic-helpers.h"
 #include "common/match.h"
+#include "common/mem.h"
 #include "common/string-helpers.h"
 #include "config/rcxml.h"
 #include "button/button-png.h"
@@ -48,8 +47,25 @@ struct button {
 	} active, inactive;
 };
 
+enum corner {
+	LAB_CORNER_UNKNOWN = 0,
+	LAB_CORNER_TOP_LEFT,
+	LAB_CORNER_TOP_RIGHT,
+};
+
+struct rounded_corner_ctx {
+	struct wlr_box *box;
+	double radius;
+	double line_width;
+	float *fill_color;
+	float *border_color;
+	enum corner corner;
+};
+
+static struct lab_data_buffer *rounded_rect(struct rounded_corner_ctx *ctx);
+
 static void
-drop(struct lab_data_buffer **buffer)
+zdrop(struct lab_data_buffer **buffer)
 {
 	if (*buffer) {
 		wlr_buffer_drop(&(*buffer)->base);
@@ -57,10 +73,29 @@ drop(struct lab_data_buffer **buffer)
 	}
 }
 
+static enum corner
+corner_from_icon_name(const char *icon_name)
+{
+	assert(icon_name);
+
+	/*
+	 * TODO: Once we implement titleLayout we can make the
+	 *       return values depend on parsed config values.
+	 */
+	if (!strcmp(icon_name, "menu")) {
+		return LAB_CORNER_TOP_LEFT;
+	} else if (!strcmp(icon_name, "close")) {
+		return LAB_CORNER_TOP_RIGHT;
+	}
+	return LAB_CORNER_UNKNOWN;
+}
+
 static void
-create_hover_fallback(struct theme *theme, struct lab_data_buffer **hover_buffer,
+create_hover_fallback(struct theme *theme, const char *icon_name,
+		struct lab_data_buffer **hover_buffer,
 		struct lab_data_buffer *icon_buffer)
 {
+	assert(icon_name);
 	assert(icon_buffer);
 	assert(!*hover_buffer);
 
@@ -69,7 +104,6 @@ create_hover_fallback(struct theme *theme, struct lab_data_buffer **hover_buffer
 	int icon_width = cairo_image_surface_get_width(icon.surface);
 	int icon_height = cairo_image_surface_get_height(icon.surface);
 
-	/* TODO: need to somehow respect rounded corners */
 	int width = SSD_BUTTON_WIDTH;
 	int height = theme->title_height;
 
@@ -102,9 +136,28 @@ create_hover_fallback(struct theme *theme, struct lab_data_buffer **hover_buffer
 	cairo_paint(cairo);
 
 	/* Overlay (non-multiplied alpha) */
-	set_cairo_color(cairo, (float[4]) { 0.5f, 0.5f, 0.5f, 0.3f});
-	cairo_rectangle(cairo, 0, 0, width, height);
-	cairo_fill(cairo);
+	float overlay_color[4] = { 0.5f, 0.5f, 0.5f, 0.3f};
+	enum corner corner = corner_from_icon_name(icon_name);
+
+	if (corner == LAB_CORNER_UNKNOWN) {
+		set_cairo_color(cairo, overlay_color);
+		cairo_rectangle(cairo, 0, 0, width, height);
+		cairo_fill(cairo);
+	} else {
+		struct rounded_corner_ctx rounded_ctx = {
+			.box = &(struct wlr_box) { .width = width, .height = height },
+			.radius = rc.corner_radius,
+			.line_width = theme->border_width,
+			.fill_color = overlay_color,
+			.border_color = overlay_color,
+			.corner = corner
+		};
+		struct lab_data_buffer *overlay_buffer = rounded_rect(&rounded_ctx);
+		cairo_set_source_surface(cairo,
+			cairo_get_target(overlay_buffer->cairo), 0, 0);
+		cairo_paint(cairo);
+		wlr_buffer_drop(&overlay_buffer->base);
+	}
 	cairo_surface_flush(surf);
 
 	if (icon.is_duplicate) {
@@ -222,8 +275,8 @@ load_buttons(struct theme *theme)
 	for (size_t i = 0; i < ARRAY_SIZE(buttons); ++i) {
 		struct button *b = &buttons[i];
 
-		drop(b->active.buffer);
-		drop(b->inactive.buffer);
+		zdrop(b->active.buffer);
+		zdrop(b->inactive.buffer);
 
 		/* PNG */
 		snprintf(filename, sizeof(filename), "%s-active.png", b->name);
@@ -258,7 +311,7 @@ load_buttons(struct theme *theme)
 		 * For example max_hover_toggled instead of max_toggled_hover
 		 */
 		if (b->alt_name) {
-			snprintf(filename, sizeof(filename), "%s.xbm", b->name);
+			snprintf(filename, sizeof(filename), "%s.xbm", b->alt_name);
 		}  else {
 			filename[0] = '\0';
 		}
@@ -289,7 +342,7 @@ load_buttons(struct theme *theme)
 	}
 
 	/*
-	 * If hover-icons do not exist, add fallbacks by coping the non-hover
+	 * If hover-icons do not exist, add fallbacks by copying the non-hover
 	 * variant (base) and then adding an overlay.
 	 */
 	for (size_t i = 0; i < ARRAY_SIZE(buttons); i++) {
@@ -307,12 +360,12 @@ load_buttons(struct theme *theme)
 			struct button *base = &buttons[j];
 			if (!strcmp(basename, base->name)) {
 				if (!*hover_button->active.buffer) {
-					create_hover_fallback(theme,
+					create_hover_fallback(theme, basename,
 						hover_button->active.buffer,
 						*base->active.buffer);
 				}
 				if (!*hover_button->inactive.buffer) {
-					create_hover_fallback(theme,
+					create_hover_fallback(theme, basename,
 						hover_button->inactive.buffer,
 						*base->inactive.buffer);
 				}
@@ -666,75 +719,38 @@ process_line(struct theme *theme, char *line)
 }
 
 static void
-theme_read(struct theme *theme, const char *theme_name)
+theme_read(struct theme *theme, struct wl_list *paths)
 {
-	FILE *stream = NULL;
-	char *line = NULL;
-	size_t len = 0;
-	char themerc[4096];
+	bool should_merge_config = rc.merge_config;
+	struct wl_list *(*iter)(struct wl_list *list);
+	iter = should_merge_config ? paths_get_prev : paths_get_next;
 
-	if (strlen(theme_dir(theme_name))) {
-		snprintf(themerc, sizeof(themerc), "%s/themerc",
-			theme_dir(theme_name));
-		stream = fopen(themerc, "r");
-	}
-	if (!stream) {
-		if (theme_name) {
-			wlr_log(WLR_INFO, "cannot find theme %s", theme_name);
+	for (struct wl_list *elm = iter(paths); elm != paths; elm = iter(elm)) {
+		struct path *path = wl_container_of(elm, path, link);
+		FILE *stream = fopen(path->string, "r");
+		if (!stream) {
+			continue;
 		}
-		return;
-	}
-	wlr_log(WLR_INFO, "read theme %s", themerc);
-	while (getline(&line, &len, stream) != -1) {
-		char *p = strrchr(line, '\n');
-		if (p) {
-			*p = '\0';
+
+		wlr_log(WLR_INFO, "read theme %s", path->string);
+
+		char *line = NULL;
+		size_t len = 0;
+		while (getline(&line, &len, stream) != -1) {
+			char *p = strrchr(line, '\n');
+			if (p) {
+				*p = '\0';
+			}
+			process_line(theme, line);
+		}
+		zfree(line);
+		fclose(stream);
+		if (!should_merge_config) {
+			break;
 		}
-		process_line(theme, line);
 	}
-	free(line);
-	fclose(stream);
 }
 
-static void
-theme_read_override(struct theme *theme)
-{
-	char f[4096] = { 0 };
-	snprintf(f, sizeof(f), "%s/themerc-override", rc.config_dir);
-
-	FILE *stream = fopen(f, "r");
-	if (!stream) {
-		wlr_log(WLR_INFO, "no theme override '%s'", f);
-		return;
-	}
-
-	wlr_log(WLR_INFO, "read theme-override %s", f);
-	char *line = NULL;
-	size_t len = 0;
-	while (getline(&line, &len, stream) != -1) {
-		char *p = strrchr(line, '\n');
-		if (p) {
-			*p = '\0';
-		}
-		process_line(theme, line);
-	}
-	free(line);
-	fclose(stream);
-}
-
-struct rounded_corner_ctx {
-	struct wlr_box *box;
-	double radius;
-	double line_width;
-	float *fill_color;
-	float *border_color;
-	enum {
-		LAB_CORNER_UNKNOWN = 0,
-		LAB_CORNER_TOP_LEFT,
-		LAB_CORNER_TOP_RIGHT,
-	} corner;
-};
-
 static struct lab_data_buffer *
 rounded_rect(struct rounded_corner_ctx *ctx)
 {
@@ -751,7 +767,7 @@ rounded_rect(struct rounded_corner_ctx *ctx)
 
 	struct lab_data_buffer *buffer;
 	/* TODO: scale */
-	buffer = buffer_create_cairo(w, h, 1, true);
+	buffer = buffer_create_cairo(w, h, 1, /*free_on_destroy*/ true);
 
 	cairo_t *cairo = buffer->cairo;
 	cairo_surface_t *surf = cairo_get_target(cairo);
@@ -1003,10 +1019,15 @@ theme_init(struct theme *theme, const char *theme_name)
 	theme_builtin(theme);
 
 	/* Read <data-dir>/share/themes/$theme_name/openbox-3/themerc */
-	theme_read(theme, theme_name);
+	struct wl_list paths;
+	paths_theme_create(&paths, theme_name, "themerc");
+	theme_read(theme, &paths);
+	paths_destroy(&paths);
 
 	/* Read <config-dir>/labwc/themerc-override */
-	theme_read_override(theme);
+	paths_config_create(&paths, "themerc-override");
+	theme_read(theme, &paths);
+	paths_destroy(&paths);
 
 	post_processing(theme);
 	create_corners(theme);
@@ -1016,12 +1037,8 @@ theme_init(struct theme *theme, const char *theme_name)
 void
 theme_finish(struct theme *theme)
 {
-	wlr_buffer_drop(&theme->corner_top_left_active_normal->base);
-	wlr_buffer_drop(&theme->corner_top_left_inactive_normal->base);
-	wlr_buffer_drop(&theme->corner_top_right_active_normal->base);
-	wlr_buffer_drop(&theme->corner_top_right_inactive_normal->base);
-	theme->corner_top_left_active_normal = NULL;
-	theme->corner_top_left_inactive_normal = NULL;
-	theme->corner_top_right_active_normal = NULL;
-	theme->corner_top_right_inactive_normal = NULL;
+	zdrop(&theme->corner_top_left_active_normal);
+	zdrop(&theme->corner_top_left_inactive_normal);
+	zdrop(&theme->corner_top_right_active_normal);
+	zdrop(&theme->corner_top_right_inactive_normal);
 }
