@@ -9,6 +9,7 @@
 #include "input/keyboard.h"
 #include "labwc.h"
 #include "menu/menu.h"
+#include "placement.h"
 #include "regions.h"
 #include "resize_indicator.h"
 #include "snap.h"
@@ -192,7 +193,7 @@ view_is_focusable_from(struct view *view, struct wlr_surface *prev)
  * They may be called repeatably during output layout changes.
  */
 
-static enum view_edge
+enum view_edge
 view_edge_invert(enum view_edge edge)
 {
 	switch (edge) {
@@ -251,14 +252,38 @@ view_get_edge_snap_box(struct view *view, struct output *output,
 	return dst;
 }
 
-static void
-view_discover_output(struct view *view)
+static bool
+view_discover_output(struct view *view, struct wlr_box *geometry)
 {
 	assert(view);
 	assert(!view->fullscreen);
-	view->output = output_nearest_to(view->server,
-		view->current.x + view->current.width / 2,
-		view->current.y + view->current.height / 2);
+
+	if (!geometry) {
+		geometry = &view->current;
+	}
+
+	struct output *output =
+		output_nearest_to(view->server,
+			geometry->x + geometry->width / 2,
+			geometry->y + geometry->height / 2);
+
+	if (output && output != view->output) {
+		view->output = output;
+		return true;
+	}
+
+	return false;
+}
+
+static void
+set_adaptive_sync_fullscreen(struct view *view)
+{
+	if (rc.adaptive_sync != LAB_ADAPTIVE_SYNC_FULLSCREEN) {
+		return;
+	}
+	/* Enable adaptive sync if view is fullscreen */
+	output_enable_adaptive_sync(view->output->wlr_output, view->fullscreen);
+	wlr_output_commit(view->output->wlr_output);
 }
 
 void
@@ -284,6 +309,7 @@ view_set_activated(struct view *view, bool activated)
 			keyboard_update_layout(&view->server->seat, view->keyboard_layout);
 		}
 	}
+	set_adaptive_sync_fullscreen(view);
 }
 
 void
@@ -307,6 +333,34 @@ view_close(struct view *view)
 	}
 }
 
+static void
+view_update_outputs(struct view *view)
+{
+	struct output *output;
+	struct wlr_output_layout *layout = view->server->output_layout;
+
+	view->outputs = 0;
+	wl_list_for_each(output, &view->server->outputs, link) {
+		if (output_is_usable(output) && wlr_output_layout_intersects(
+				layout, output->wlr_output, &view->current)) {
+			view->outputs |= (1ull << output->scene_output->index);
+		}
+	}
+
+	if (view->toplevel.handle) {
+		foreign_toplevel_update_outputs(view);
+	}
+}
+
+bool
+view_on_output(struct view *view, struct output *output)
+{
+	assert(view);
+	assert(output);
+	return output->scene_output
+			&& (view->outputs & (1ull << output->scene_output->index));
+}
+
 void
 view_move(struct view *view, int x, int y)
 {
@@ -330,13 +384,11 @@ view_moved(struct view *view)
 	 * output when they enter that state.
 	 */
 	if (view_is_floating(view)) {
-		view_discover_output(view);
+		view_discover_output(view, NULL);
 	}
+	view_update_outputs(view);
 	ssd_update_geometry(view->ssd);
 	cursor_update_focus(view->server);
-	if (view->toplevel.handle) {
-		foreign_toplevel_update_outputs(view);
-	}
 	if (rc.resize_indicator && view->server->grabbed_view == view) {
 		resize_indicator_update(view);
 	}
@@ -358,6 +410,7 @@ view_resize_relative(struct view *view, int left, int right, int top, int bottom
 	if (view->fullscreen || view->maximized != VIEW_AXIS_NONE) {
 		return;
 	}
+	view_set_shade(view, false);
 	struct wlr_box newgeo = view->pending;
 	newgeo.x -= left;
 	newgeo.width += left + right;
@@ -407,18 +460,23 @@ view_move_to_cursor(struct view *view)
 	int y = view->server->seat.cursor->y - (geo.height / 2);
 
 	struct wlr_box usable = output_usable_area_in_layout_coords(pending_output);
+
+	/* Limit usable region to account for gap */
+	usable.x += rc.gap;
+	usable.y += rc.gap;
+	usable.width -= 2 * rc.gap;
+	usable.height -= 2 * rc.gap;
+
 	if (x + geo.width > usable.x + usable.width) {
 		x = usable.x + usable.width - geo.width;
 	}
-	x = MAX(x, usable.x);
+	x = MAX(x, usable.x) + margin.left;
 
 	if (y + geo.height > usable.y + usable.height) {
 		y = usable.y + usable.height - geo.height;
 	}
-	y = MAX(y, usable.y);
+	y = MAX(y, usable.y) + margin.top;
 
-	x += margin.left;
-	y += margin.top;
 	view_move(view, x, y);
 }
 
@@ -584,10 +642,12 @@ view_compute_centered_position(struct view *view, const struct wlr_box *ref,
 	return true;
 }
 
-bool
-view_adjust_floating_geometry(struct view *view, struct wlr_box *geometry)
+static bool
+adjust_floating_geometry(struct view *view, struct wlr_box *geometry,
+		bool midpoint_visibility)
 {
 	assert(view);
+
 	if (!output_is_usable(view->output)) {
 		wlr_log(WLR_ERROR, "view has no output, not positioning");
 		return false;
@@ -600,16 +660,10 @@ view_adjust_floating_geometry(struct view *view, struct wlr_box *geometry)
 	}
 
 	bool adjusted = false;
-	/*
-	 * First check whether the view is onscreen. For now, "onscreen"
-	 * is defined as even one pixel of the client area being visible.
-	 */
+	bool onscreen = false;
 	if (wlr_output_layout_intersects(view->server->output_layout,
-			NULL, geometry)) {
-		/*
-		 * If onscreen, then make sure the titlebar is also
-		 * visible (and not overlapping any panels/docks)
-		 */
+			view->output->wlr_output, geometry)) {
+		/* Always make sure the titlebar starts within the usable area */
 		struct border margin = ssd_get_margin(view->ssd);
 		struct wlr_box usable =
 			output_usable_area_in_layout_coords(view->output);
@@ -618,17 +672,43 @@ view_adjust_floating_geometry(struct view *view, struct wlr_box *geometry)
 			geometry->x = usable.x + margin.left;
 			adjusted = true;
 		}
+
 		if (geometry->y < usable.y + margin.top) {
 			geometry->y = usable.y + margin.top;
 			adjusted = true;
 		}
-	} else {
-		/* If offscreen, then just center the view */
-		adjusted = view_compute_centered_position(view, NULL,
-			geometry->width, geometry->height,
-			&geometry->x, &geometry->y);
+
+		if (!midpoint_visibility) {
+			/*
+			 * If midpoint visibility is not required, the view is
+			 * on screen if at least one pixel is visible.
+			 */
+			onscreen = true;
+		} else {
+			/* Otherwise, make sure the midpoint is on screen */
+			int mx = geometry->x + geometry->width / 2;
+			int my = geometry->y + geometry->height / 2;
+
+			onscreen = mx <= usable.x + usable.width &&
+				my <= usable.y + usable.height;
+		}
 	}
-	return adjusted;
+
+	if (onscreen) {
+		return adjusted;
+	}
+
+	/* Reposition offscreen automatically if configured to do so */
+	if (rc.placement_policy == LAB_PLACE_AUTOMATIC) {
+		if (placement_find_best(view, geometry)) {
+			return true;
+		}
+	}
+
+	/* If automatic placement failed or was not enabled, just center */
+	return view_compute_centered_position(view, NULL,
+		geometry->width, geometry->height,
+		&geometry->x, &geometry->y);
 }
 
 static void
@@ -667,6 +747,18 @@ view_store_natural_geometry(struct view *view)
 	}
 }
 
+int
+view_effective_height(struct view *view, bool use_pending)
+{
+	assert(view);
+
+	if (view->shaded) {
+		return 0;
+	}
+
+	return use_pending ? view->pending.height : view->current.height;
+}
+
 void
 view_center(struct view *view, const struct wlr_box *ref)
 {
@@ -678,6 +770,69 @@ view_center(struct view *view, const struct wlr_box *ref)
 	}
 }
 
+void
+view_place_initial(struct view *view, bool allow_cursor)
+{
+	if (allow_cursor && rc.placement_policy == LAB_PLACE_CURSOR) {
+		view_move_to_cursor(view);
+		return;
+	} else if (rc.placement_policy == LAB_PLACE_AUTOMATIC) {
+		struct wlr_box geometry = view->pending;
+		if (placement_find_best(view, &geometry)) {
+			view_move(view, geometry.x, geometry.y);
+			return;
+		}
+	}
+
+	view_center(view, NULL);
+}
+
+void
+view_constrain_size_to_that_of_usable_area(struct view *view)
+{
+	if (!view || !view->output || view->fullscreen) {
+		return;
+	}
+
+	struct wlr_box usable_area =
+			output_usable_area_in_layout_coords(view->output);
+	struct border margin = ssd_get_margin(view->ssd);
+
+	int available_width = usable_area.width - margin.left - margin.right;
+	int available_height = usable_area.height - margin.top - margin.bottom;
+
+	if (available_width <= 0 || available_height <= 0) {
+		return;
+	}
+
+	if (available_height >= view->pending.height &&
+			available_width >= view->pending.width) {
+		return;
+	}
+
+	int width = MIN(view->pending.width, available_width);
+	int height = MIN(view->pending.height, available_height);
+
+	int right_edge = usable_area.x + usable_area.width;
+	int bottom_edge = usable_area.y + usable_area.height;
+
+	int x =
+		MAX(usable_area.x + margin.left,
+			MIN(view->pending.x, right_edge - width - margin.right));
+
+	int y =
+		MAX(usable_area.y + margin.top,
+			MIN(view->pending.y, bottom_edge - height - margin.bottom));
+
+	struct wlr_box box = {
+		.x = x,
+		.y = y,
+		.width = width,
+		.height = height,
+	};
+	view_move_resize(view, box);
+}
+
 static void
 view_apply_natural_geometry(struct view *view)
 {
@@ -685,7 +840,8 @@ view_apply_natural_geometry(struct view *view)
 	assert(view_is_floating(view));
 
 	struct wlr_box geometry = view->natural_geometry;
-	view_adjust_floating_geometry(view, &geometry);
+	adjust_floating_geometry(view, &geometry,
+		/* midpoint_visibility */ false);
 	view_move_resize(view, geometry);
 }
 
@@ -918,6 +1074,15 @@ view_is_floating(struct view *view)
 		|| view_is_tiled(view));
 }
 
+static void
+view_notify_tiled(struct view *view)
+{
+	assert(view);
+	if (view->impl->notify_tiled) {
+		view->impl->notify_tiled(view);
+	}
+}
+
 /* Reset tiled state of view without changing geometry */
 void
 view_set_untiled(struct view *view)
@@ -926,6 +1091,7 @@ view_set_untiled(struct view *view)
 	view->tiled = VIEW_EDGE_INVALID;
 	view->tiled_region = NULL;
 	zfree(view->tiled_region_evacuate);
+	view_notify_tiled(view);
 }
 
 void
@@ -933,12 +1099,17 @@ view_maximize(struct view *view, enum view_axis axis,
 		bool store_natural_geometry)
 {
 	assert(view);
+
 	if (view->maximized == axis) {
 		return;
 	}
+
 	if (view->fullscreen) {
 		return;
 	}
+
+	view_set_shade(view, false);
+
 	if (axis != VIEW_AXIS_NONE) {
 		/*
 		 * Maximize via keybind or client request cancels
@@ -948,6 +1119,7 @@ view_maximize(struct view *view, enum view_axis axis,
 		interactive_cancel(view);
 		if (store_natural_geometry && view_is_floating(view)) {
 			view_store_natural_geometry(view);
+			view_invalidate_last_layout_geometry(view);
 		}
 	}
 	set_maximized(view, axis);
@@ -987,6 +1159,12 @@ void
 view_toggle_decorations(struct view *view)
 {
 	assert(view);
+
+	/* Reject decoration toggles when shaded */
+	if (view->shaded) {
+		return;
+	}
+
 	if (rc.ssd_keep_border && view->ssd_enabled && view->ssd
 			&& !view->ssd_titlebar_hidden) {
 		/*
@@ -1109,6 +1287,7 @@ void
 view_toggle_fullscreen(struct view *view)
 {
 	assert(view);
+
 	view_set_fullscreen(view, !view->fullscreen);
 }
 
@@ -1116,6 +1295,11 @@ view_toggle_fullscreen(struct view *view)
 static void
 set_fullscreen(struct view *view, bool fullscreen)
 {
+	/* When going fullscreen, unshade the window */
+	if (fullscreen) {
+		view_set_shade(view, false);
+	}
+
 	/* Hide decorations when going fullscreen */
 	if (fullscreen && view->ssd_enabled) {
 		undecorate(view);
@@ -1160,6 +1344,7 @@ view_set_fullscreen(struct view *view, bool fullscreen)
 		 */
 		interactive_cancel(view);
 		view_store_natural_geometry(view);
+		view_invalidate_last_layout_geometry(view);
 	}
 
 	set_fullscreen(view, fullscreen);
@@ -1168,6 +1353,70 @@ view_set_fullscreen(struct view *view, bool fullscreen)
 	} else {
 		view_apply_special_geometry(view);
 	}
+	set_adaptive_sync_fullscreen(view);
+}
+
+static bool
+last_layout_geometry_is_valid(struct view *view)
+{
+	return view->last_layout_geometry.width > 0
+		&& view->last_layout_geometry.height > 0;
+}
+
+static void
+update_last_layout_geometry(struct view *view)
+{
+	/*
+	 * Only update an invalid last-layout geometry to prevent a series of
+	 * successive layout changes from continually replacing the "preferred"
+	 * location with whatever location the view currently holds. The
+	 * "preferred" location should be whatever state was set by user
+	 * interaction, not automatic responses to layout changes.
+	 */
+	if (last_layout_geometry_is_valid(view)) {
+		return;
+	}
+
+	if (view_is_floating(view)) {
+		view->last_layout_geometry = view->pending;
+	} else {
+		view->last_layout_geometry = view->natural_geometry;
+	}
+}
+
+static bool
+apply_last_layout_geometry(struct view *view, bool force_update)
+{
+	/* Only apply a valid last-layout geometry */
+	if (!last_layout_geometry_is_valid(view)) {
+		return false;
+	}
+
+	/*
+	 * Unless forced, the last-layout geometry is only applied
+	 * when the relevant view geometry is distinct.
+	 */
+	if (!force_update) {
+		struct wlr_box *relevant = view_is_floating(view) ?
+			&view->pending : &view->natural_geometry;
+
+		if (wlr_box_equal(relevant, &view->last_layout_geometry)) {
+			return false;
+		}
+	}
+
+	view->natural_geometry = view->last_layout_geometry;
+	adjust_floating_geometry(view, &view->natural_geometry,
+		/* midpoint_visibility */ true);
+	return true;
+}
+
+void
+view_invalidate_last_layout_geometry(struct view *view)
+{
+	assert(view);
+	view->last_layout_geometry.width = 0;
+	view->last_layout_geometry.height = 0;
 }
 
 void
@@ -1175,35 +1424,76 @@ view_adjust_for_layout_change(struct view *view)
 {
 	assert(view);
 
-	/* Exit fullscreen if output is lost */
 	bool was_fullscreen = view->fullscreen;
-	if (was_fullscreen && !output_is_usable(view->output)) {
-		set_fullscreen(view, false);
+	bool is_floating = view_is_floating(view);
+
+	if (!output_is_usable(view->output)) {
+		/* A view losing an output should have a last-layout geometry */
+		update_last_layout_geometry(view);
+
+		/* Exit fullscreen and re-assess floating status */
+		if (was_fullscreen) {
+			set_fullscreen(view, false);
+			is_floating = view_is_floating(view);
+		}
+	}
+
+	/* Restore any full-screen window to natural geometry */
+	bool use_natural = was_fullscreen;
+
+	/* Capture a pointer to the last-layout geometry (only if valid) */
+	struct wlr_box *last_geometry = NULL;
+	if (last_layout_geometry_is_valid(view)) {
+		last_geometry = &view->last_layout_geometry;
 	}
 
 	/*
-	 * Floating views are always assigned to the nearest output.
-	 * Maximized/tiled views remain on the same output if possible.
+	 * Check if an output change is required:
+	 * - Floating views are always mapped to the nearest output
+	 * - Any view without a usable output needs to be repositioned
+	 * - Any view with a valid last-layout geometry might be better
+	 *   positioned on another output
 	 */
-	bool is_floating = view_is_floating(view);
-	if (is_floating || !output_is_usable(view->output)) {
-		view_discover_output(view);
+	if (is_floating || last_geometry || !output_is_usable(view->output)) {
+		/* Move the view to an appropriate output, if needed */
+		bool output_changed = view_discover_output(view, last_geometry);
+
+		/*
+		 * Try to apply the last-layout to the natural geometry
+		 * (adjusting to ensure that it fits on the screen). This is
+		 * forced if the output has changed, but will be done
+		 * opportunistically even on the same output if the last-layout
+		 * geometry is different from the view's governing geometry.
+		 */
+		if (apply_last_layout_geometry(view, output_changed)) {
+			use_natural = true;
+		}
+
+		/*
+		 * Whether or not the view has moved, the layout has changed.
+		 * Ensure that the view now has a valid last-layout geometry.
+		 */
+		update_last_layout_geometry(view);
 	}
 
 	if (!is_floating) {
 		view_apply_special_geometry(view);
-	} else if (was_fullscreen) {
+	} else if (use_natural) {
+		/*
+		 * Move the window to its natural location, either because it
+		 * was fullscreen or we are trying to restore a prior layout.
+		 */
 		view_apply_natural_geometry(view);
 	} else {
-		/* reposition view if it's offscreen */
+		/* Otherwise, just ensure the view is on screen. */
 		struct wlr_box geometry = view->pending;
-		if (view_adjust_floating_geometry(view, &geometry)) {
+		if (adjust_floating_geometry(view, &geometry,
+					/* midpoint_visibility */ true)) {
 			view_move_resize(view, geometry);
 		}
 	}
-	if (view->toplevel.handle) {
-		foreign_toplevel_update_outputs(view);
-	}
+
+	view_update_outputs(view);
 }
 
 void
@@ -1229,7 +1519,7 @@ view_on_output_destroy(struct view *view)
 	view->output = NULL;
 }
 
-static struct output *
+struct output *
 view_get_adjacent_output(struct view *view, enum view_edge edge)
 {
 	assert(view);
@@ -1279,6 +1569,42 @@ view_get_adjacent_output(struct view *view, enum view_edge edge)
 	return output;
 }
 
+static int
+shift_view_to_usable_1d(int size,
+		int cur_pos, int cur_lo, int cur_extent,
+		int next_pos, int next_lo, int next_extent,
+		int margin_lo, int margin_hi)
+{
+	int cur_min = cur_lo + rc.gap + margin_lo;
+	int cur_max = cur_lo + cur_extent - rc.gap - margin_hi;
+
+	int next_min = next_lo + rc.gap + margin_lo;
+	int next_max = next_lo + next_extent - rc.gap - margin_hi;
+
+	/*
+	 * If the view is fully within the usable area of its original display,
+	 * ensure that it is also fully within the usable area of the target.
+	 */
+	if (cur_pos >= cur_min && cur_pos + size <= cur_max) {
+		if (next_pos >= next_min && next_pos + size > next_max) {
+			next_pos = next_max - size;
+		}
+
+		return MAX(next_pos, next_min);
+	}
+
+	/*
+	 * If the view was not fully within the usable area of its original
+	 * display, kick it onscreen if its midpoint will be off the target.
+	 */
+	int midpoint = next_pos + size / 2;
+	if (next_pos >= next_min && midpoint > next_lo + next_extent) {
+		next_pos = next_max - size;
+	}
+
+	return MAX(next_pos, next_min);
+}
+
 void
 view_move_to_edge(struct view *view, enum view_edge direction, bool snap_to_windows)
 {
@@ -1289,27 +1615,7 @@ view_move_to_edge(struct view *view, enum view_edge direction, bool snap_to_wind
 	}
 
 	int dx = 0, dy = 0;
-	if (snap_to_windows) {
-		snap_vector_to_next_edge(view, direction, &dx, &dy);
-	} else {
-		struct border distance = snap_get_max_distance(view);
-		switch (direction) {
-		case VIEW_EDGE_LEFT:
-			dx = distance.left;
-			break;
-		case VIEW_EDGE_UP:
-			dy = distance.top;
-			break;
-		case VIEW_EDGE_RIGHT:
-			dx = distance.right;
-			break;
-		case VIEW_EDGE_DOWN:
-			dy = distance.bottom;
-			break;
-		default:
-			return;
-		}
-	}
+	snap_move_to_edge(view, direction, snap_to_windows, &dx, &dy);
 
 	if (dx != 0 || dy != 0) {
 		/* Move the window if a change was discovered */
@@ -1355,30 +1661,26 @@ view_move_to_edge(struct view *view, enum view_edge direction, bool snap_to_wind
 		destination_y = top;
 		break;
 	case VIEW_EDGE_DOWN:
-		destination_y = bottom - view->pending.height;
+		destination_y = bottom
+			- view_effective_height(view, /* use_pending */ true);
 		break;
 	default:
 		return;
 	}
 
-	/* If more than half the view is right of usable region, align to right */
-	int midpoint = destination_x + view->pending.width / 2;
+	struct wlr_box original_usable =
+		output_usable_area_in_layout_coords(view->output);
 
-	if (destination_x >= left && midpoint > usable.x + usable.width) {
-		destination_x = right - view->pending.width;
-	}
+	/* Make sure the window is appropriately in view along the x direction */
+	destination_x = shift_view_to_usable_1d(view->pending.width,
+		view->pending.x, original_usable.x, original_usable.width,
+		destination_x, usable.x, usable.width, margin.left, margin.right);
 
-	/* Never allow the window to start left of the usable edge */
-	destination_x = MAX(destination_x, left);
-
-	/* If more than half the view is below usable region, align to bottom */
-	midpoint = destination_y + view->pending.height / 2;
-	if (destination_y >= top && midpoint > usable.y + usable.height) {
-		destination_y = bottom - view->pending.height;
-	}
-
-	/* Never allow the window to start above the usable edge */
-	destination_y = MAX(destination_y, top);
+	/* Make sure the window is appropriately in view along the y direction */
+	int eff_height = view_effective_height(view, /* use_pending */ true);
+	destination_y = shift_view_to_usable_1d(eff_height,
+		view->pending.y, original_usable.y, original_usable.height,
+		destination_y, usable.y, usable.height, margin.top, margin.bottom);
 
 	view_set_untiled(view);
 	view_set_output(view, output);
@@ -1393,12 +1695,15 @@ view_grow_to_edge(struct view *view, enum view_edge direction)
 	if (view->fullscreen || view->maximized != VIEW_AXIS_NONE) {
 		return;
 	}
+
 	if (!output_is_usable(view->output)) {
 		wlr_log(WLR_ERROR, "view has no output, not growing view");
 		return;
 	}
 
-	struct wlr_box geo = view->pending;
+	view_set_shade(view, false);
+
+	struct wlr_box geo;
 	snap_grow_to_next_edge(view, direction, &geo);
 	view_move_resize(view, geo);
 }
@@ -1407,15 +1712,19 @@ void
 view_shrink_to_edge(struct view *view, enum view_edge direction)
 {
 	assert(view);
+
 	/* TODO: allow shrink to edge if maximized along the other axis */
 	if (view->fullscreen || view->maximized != VIEW_AXIS_NONE) {
 		return;
 	}
+
 	if (!output_is_usable(view->output)) {
 		wlr_log(WLR_ERROR, "view has no output, not shrinking view");
 		return;
 	}
 
+	view_set_shade(view, false);
+
 	struct wlr_box geo = view->pending;
 	snap_shrink_to_next_edge(view, direction, &geo);
 	view_move_resize(view, geo);
@@ -1464,15 +1773,19 @@ view_snap_to_edge(struct view *view, enum view_edge edge,
 			bool across_outputs, bool store_natural_geometry)
 {
 	assert(view);
+
 	if (view->fullscreen) {
 		return;
 	}
+
 	struct output *output = view->output;
 	if (!output_is_usable(output)) {
 		wlr_log(WLR_ERROR, "view has no output, not snapping to edge");
 		return;
 	}
 
+	view_set_shade(view, false);
+
 	if (across_outputs && view->tiled == edge && view->maximized == VIEW_AXIS_NONE) {
 		/* We are already tiled for this edge; try to switch outputs */
 		output = view_get_adjacent_output(view, edge);
@@ -1501,10 +1814,12 @@ view_snap_to_edge(struct view *view, enum view_edge edge,
 	} else if (store_natural_geometry) {
 		/* store current geometry as new natural_geometry */
 		view_store_natural_geometry(view);
+		view_invalidate_last_layout_geometry(view);
 	}
 	view_set_untiled(view);
 	view_set_output(view, output);
 	view->tiled = edge;
+	view_notify_tiled(view);
 	view_apply_tiled_geometry(view);
 }
 
@@ -1514,15 +1829,19 @@ view_snap_to_region(struct view *view, struct region *region,
 {
 	assert(view);
 	assert(region);
+
 	if (view->fullscreen) {
 		return;
 	}
+
 	/* view_apply_region_geometry() needs a usable output */
 	if (!output_is_usable(view->output)) {
 		wlr_log(WLR_ERROR, "view has no output, not snapping to region");
 		return;
 	}
 
+	view_set_shade(view, false);
+
 	if (view->maximized != VIEW_AXIS_NONE) {
 		/* Unmaximize + keep using existing natural_geometry */
 		view_maximize(view, VIEW_AXIS_NONE,
@@ -1530,12 +1849,39 @@ view_snap_to_region(struct view *view, struct region *region,
 	} else if (store_natural_geometry) {
 		/* store current geometry as new natural_geometry */
 		view_store_natural_geometry(view);
+		view_invalidate_last_layout_geometry(view);
 	}
 	view_set_untiled(view);
 	view->tiled_region = region;
+	view_notify_tiled(view);
 	view_apply_region_geometry(view);
 }
 
+void
+view_move_to_output(struct view *view, struct output *output)
+{
+	assert(view);
+	if (view->fullscreen) {
+		return;
+	}
+
+	view_invalidate_last_layout_geometry(view);
+	view_set_output(view, output);
+	if (view_is_floating(view)) {
+		struct wlr_box output_area = output_usable_area_in_layout_coords(output);
+		view->pending.x = output_area.x;
+		view->pending.y = output_area.y;
+		view_place_initial(view, /* allow_cursor */ false);
+	} else if (view->maximized != VIEW_AXIS_NONE) {
+		view_apply_maximized_geometry(view);
+	} else if (view->tiled) {
+		view_apply_tiled_geometry(view);
+	} else if (view->tiled_region) {
+		struct region *region = regions_from_name(view->tiled_region->name, output);
+		view_snap_to_region(view, region, /*store_natural_geometry*/ false);
+	}
+}
+
 static void
 for_each_subview(struct view *view, void (*action)(struct view *))
 {
@@ -1766,12 +2112,35 @@ view_connect_map(struct view *view, struct wlr_surface *surface)
 	mappable_connect(&view->mappable, surface, handle_map, handle_unmap);
 }
 
+void
+view_set_shade(struct view *view, bool shaded)
+{
+	assert(view);
+
+	if (view->shaded == shaded) {
+		return;
+	}
+
+	/* Views without a title-bar or SSD cannot be shaded */
+	if (shaded && (!view->ssd || view->ssd_titlebar_hidden)) {
+		return;
+	}
+
+	/* If this window is being resized, cancel the resize when shading */
+	if (shaded && view->server->input_mode == LAB_INPUT_STATE_RESIZE) {
+		interactive_cancel(view);
+	}
+
+	view->shaded = shaded;
+	ssd_enable_shade(view->ssd, view->shaded);
+	wlr_scene_node_set_enabled(view->scene_node, !view->shaded);
+}
+
 void
 view_destroy(struct view *view)
 {
 	assert(view);
 	struct server *server = view->server;
-	bool need_cursor_update = false;
 
 	if (view->mappable.connected) {
 		mappable_disconnect(&view->mappable);
@@ -1793,13 +2162,11 @@ view_destroy(struct view *view)
 		/* Application got killed while moving around */
 		server->input_mode = LAB_INPUT_STATE_PASSTHROUGH;
 		server->grabbed_view = NULL;
-		need_cursor_update = true;
 		regions_hide_overlay(&server->seat);
 	}
 
 	if (server->active_view == view) {
 		server->active_view = NULL;
-		need_cursor_update = true;
 	}
 
 	if (server->last_raised_view == view) {
@@ -1822,11 +2189,6 @@ view_destroy(struct view *view)
 	osd_on_view_destroy(view);
 	undecorate(view);
 
-	if (view->scene_tree) {
-		wlr_scene_node_destroy(&view->scene_tree->node);
-		view->scene_tree = NULL;
-	}
-
 	/*
 	 * The layer-shell top-layer is disabled when an application is running
 	 * in fullscreen mode, so if that's the case, we may have to re-enable
@@ -1835,6 +2197,9 @@ view_destroy(struct view *view)
 	if (view->fullscreen && view->output) {
 		view->fullscreen = false;
 		desktop_update_top_layer_visiblity(server);
+		if (rc.adaptive_sync == LAB_ADAPTIVE_SYNC_FULLSCREEN) {
+			set_adaptive_sync_fullscreen(view);
+		}
 	}
 
 	/* If we spawned a window menu, close it */
@@ -1843,11 +2208,19 @@ view_destroy(struct view *view)
 		menu_close_root(server);
 	}
 
+	/*
+	 * Destroy the view's scene tree. View methods assume this is non-NULL,
+	 * so we should avoid any calls to those between this and freeing the
+	 * view.
+	 */
+	if (view->scene_tree) {
+		wlr_scene_node_destroy(&view->scene_tree->node);
+		view->scene_tree = NULL;
+	}
+
 	/* Remove view from server->views */
 	wl_list_remove(&view->link);
 	free(view);
 
-	if (need_cursor_update) {
-		cursor_update_focus(server);
-	}
+	cursor_update_focus(server);
 }
