@@ -60,14 +60,16 @@ _osd_update(struct server *server)
 	/* Settings */
 	uint16_t margin = 10;
 	uint16_t padding = 2;
-	uint16_t rect_height = 20;
-	uint16_t rect_width = 20;
+	uint16_t rect_height = theme->osd_workspace_switcher_boxes_height;
+	uint16_t rect_width = theme->osd_workspace_switcher_boxes_width;
+	bool hide_boxes = theme->osd_workspace_switcher_boxes_width == 0 ||
+		theme->osd_workspace_switcher_boxes_height == 0;
 
 	/* Dimensions */
 	size_t workspace_count = wl_list_length(&server->workspaces);
 	uint16_t marker_width = workspace_count * (rect_width + padding) - padding;
 	uint16_t width = margin * 2 + (marker_width < 200 ? 200 : marker_width);
-	uint16_t height = margin * 3 + rect_height + font_height(&rc.font_osd);
+	uint16_t height = margin * (hide_boxes ? 2 : 3) + rect_height + font_height(&rc.font_osd);
 
 	cairo_t *cairo;
 	cairo_surface_t *surface;
@@ -100,19 +102,23 @@ _osd_update(struct server *server)
 		};
 		draw_cairo_border(cairo, fbox, theme->osd_border_width);
 
-		uint16_t x = (width - marker_width) / 2;
-		wl_list_for_each(workspace, &server->workspaces, link) {
-			bool active =  workspace == server->workspace_current;
-			set_cairo_color(cairo, server->theme->osd_label_text_color);
-			cairo_rectangle(cairo, x, margin,
-				rect_width - padding, rect_height);
-			cairo_stroke(cairo);
-			if (active) {
+		/* Boxes */
+		uint16_t x;
+		if (!hide_boxes) {
+			x = (width - marker_width) / 2;
+			wl_list_for_each(workspace, &server->workspaces, link) {
+				bool active =  workspace == server->workspace_current;
+				set_cairo_color(cairo, server->theme->osd_label_text_color);
 				cairo_rectangle(cairo, x, margin,
 					rect_width - padding, rect_height);
-				cairo_fill(cairo);
+				cairo_stroke(cairo);
+				if (active) {
+					cairo_rectangle(cairo, x, margin,
+						rect_width - padding, rect_height);
+					cairo_fill(cairo);
+				}
+				x += rect_width + padding;
 			}
-			x += rect_width + padding;
 		}
 
 		/* Text */
@@ -127,7 +133,11 @@ _osd_update(struct server *server)
 		/* Center workspace indicator on the x axis */
 		x = font_width(&rc.font_osd, server->workspace_current->name);
 		x = (width - x) / 2;
-		cairo_move_to(cairo, x, margin * 2 + rect_height);
+		if (!hide_boxes) {
+			cairo_move_to(cairo, x, margin * 2 + rect_height);
+		} else {
+			cairo_move_to(cairo, x, (height - font_height(&rc.font_osd)) / 2.0);
+		}
 		//pango_font_description_set_weight(desc, PANGO_WEIGHT_BOLD);
 		pango_layout_set_font_description(layout, desc);
 		pango_font_description_free(desc);
@@ -276,6 +286,16 @@ workspaces_switch_to(struct workspace *target, bool update_focus)
 	wlr_scene_node_set_enabled(
 		&server->workspace_current->tree->node, false);
 
+	/* Move Omnipresent views to new workspace */
+	struct view *view;
+	enum lab_view_criteria criteria =
+		LAB_VIEW_CRITERIA_CURRENT_WORKSPACE;
+	for_each_view(view, &server->views, criteria) {
+		if (view->visible_on_all_workspaces) {
+			view_move_to_workspace(view, target);
+		}
+	}
+
 	/* Enable the new workspace */
 	wlr_scene_node_set_enabled(&target->tree->node, true);
 
@@ -290,7 +310,7 @@ workspaces_switch_to(struct workspace *target, bool update_focus)
 	 * Only refocus if the focus is not already on an always-on-top view.
 	 */
 	if (update_focus) {
-		struct view *view = server->focused_view;
+		struct view *view = server->active_view;
 		if (!view || !view_is_always_on_top(view)) {
 			desktop_focus_topmost_view(server);
 		}
